import os
import configparser

def getSetupIni():

    file = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'setup.ini')
    parser = configparser.RawConfigParser()
    parser.read( file )

    version = "0.0.1"
    name = "cardholder"
    title = 'CardHolder'

    try:
        version = parser.get("DEFAULT", 'version')
        name = parser.get("DEFAULT", 'name')
        title = parser.get("DEFAULT", 'title')
    except (configparser.NoSectionError, configparser.NoOptionError):
        pass

    return dict([ 
        ('name', name), 
        ('title', title), 
        ('version', version)
    ])




#
WINDOW_WIDTH = 400
WINDOW_HEIGHT = 120
IMG_SIZE = 16
LABEL_LENGTH = 120
LANGUAGE_DROPDOWN_LENGTH = "120"

#
PICTURE_WIDTH = 190
PICTURE_HEIGHT = 190
RATE_WIDTH = 30

HIERARCHY_TITLE_FONT_SIZE = 23
INFO_LABEL_WIDTH = 65
INFO_FONT_SIZE = 10
INFO_TITLE_FONT_SIZE = 18
ONE_LINE_HEIGHT = 18
FONT_TYPE = "Comic Sans MS"

COLOR_MAIN_BACKGROUND = "#73948d"
COLOR_CARD_MOVIE_BACKGROUND = "#b3d4c9"
COLOR_CARD_COLLECTOR_BACKGROUND = "#71a49b"
COLOR_CARD_BORDER_NORMAL_BACKGROUND = "#0e997a" #"#666666" #"#b3d4c9"
COLOR_CARD_BORDER_SELECTED_BACKGROUND = "#00ff00"
COLOR_CARDHOLDER_BACKGROUND = "#3f6b61"

COLOR_CARD_BORDER_MEDIA = "#b3d4c9"
COLOR_CARD_BORDER_CONTAINER = "#0e997a"

MAX_OVERLAPPED_CARDS = 5

RADIUS_CARDHOLDER = 10
RADIUS_CARD = 5
WIDTH_BORDER_CARD = 8
WIDTH_PANEL_MARGIN_TOP = 5
WIDTH_PANEL_MARGIN_BOTTOM = 5
WIDTH_PANEL_MARGIN_RIGHT = 5
WIDTH_PANEL_MARGIN_LEFT = 5

#COLOR_CARD_HOLDER_MAIN = "#73948d"

#
SECTION_RATING = 'rating'
SECTION_GENERAL = 'general'
SECTION_TITLE = 'title'
SECTION_EXTRA = 'extra'

RATING_KEY_FAVORITE = 'favorite'
RATING_KEY_NEW = 'new'
RATING_KEY_BEST = 'best'

FILE_CARD_INI = 'card.ini'

IMG_CONFIG_BUTTON = "cogwheel-button.png"
IMG_BACK_BUTTON = "back-button.png"
IMG_FOLDER_BUTTON = "folder.png"
IMG_EMPTY_BUTTON = "empty.png"
IMG_BEST_ON = "best-on.png"
IMG_BEST_OFF = "best-off.png"
IMG_NEW_ON = "new-on.png"
IMG_NEW_OFF = "new-off.png"
IMG_FAVORITE_ON = "favorite-on.png"
IMG_FAVORITE_OFF = "favorite-off.png"



import os
import configparser
from pathlib import Path

class Property(object):
       
    def __init__(self, file, writable=False, folder=None):
        self.writable = writable
        self.file = file
        self.folder = folder
        self.parser = configparser.RawConfigParser()
        
        # !!! make it CASE SENSITIVE !!! otherwise it duplicates the hit if there was a key with upper and lower cases. Now it throws an exception
        self.parser.optionxform = str

    def __write_file(self):
        
        if self.folder:
            Path(self.folder).mkdir(parents=True, exist_ok=True)
        
        with open(self.file, 'w') as configfile:
            self.parser.write(configfile)

    def get(self, section, key, default_value):

        # if not existing file
        if not os.path.exists(self.file):
            print("GET", "No file: ", self.file)
            
            self.parser[section]={key: default_value}
            self.__write_file()
        self.parser.read(self.file)

        try:
            result=self.parser.get(section,key)
        except (configparser.NoSectionError, configparser.NoOptionError) as e:
            print("GET", e, 'in ', self.file)
            if self.writable:
                self.update(section, key, default_value)
                result=self.parser.get(section,key)
            else:
                result = default_value
        return result

    def getBoolean(self, section, key, default_value):
        if not os.path.exists(self.file):
            self.parser[section]={key: default_value}
            self.__write_file()
        self.parser.read(self.file)

        try:
            result=self.parser.getboolean(section,key)
        except (configparser.NoSectionError, configparser.NoOptionError):
            if self.writable:
                self.update(section, key, default_value)
                # It is strange how it works with get/getboolean
                # Sometimes it reads boolean sometimes it reads string
                # I could not find out what is the problem
                #result=self.parser.get(section,key)
            result=default_value

        return result

    def update(self, section, key, value, source=None):
        if not os.path.exists(self.file):
            print("UPDATE", "No file: ", self.file, " Source: " + source if source else "")
            self.parser[section]={key: value}        
        else:
            self.parser.read(self.file)
            try:
                # if no section -> NoSectionError | if no key -> Create it
                self.parser.set(section, key, value)
            except configparser.NoSectionError as e:
                print("UPDATE", e, "in ", self.file, " Source: ", source )

                self.parser[section]={key: value}

        self.__write_file()


# ====================
#
# Handle dictionary
#
# ====================
class Dict( Property ):
    
    DICT_FILE_PRE = "resources"
    DICT_FILE_EXT = "properties"
    DICT_FOLDER = "dict"
    DICT_SECTION = "dict"

    __instance = None

    def __new__(cls):
        if cls.__instance == None:
            cls.__instance = super().__new__(cls)
        return cls.__instance

    @classmethod
    def get_instance(cls, lng):
        inst = cls.__new__(cls)
        cls.__init__(cls.__instance, lng)     
        return inst
        
    def __init__(self, lng):
        file = os.path.join(os.path.dirname(os.path.abspath(__file__)), self.__class__.DICT_FOLDER, self.__class__.DICT_FILE_PRE + "_" + lng + "." + self.__class__.DICT_FILE_EXT)
        super().__init__( file )
    
    def _(self, key):
        return self.get(self.__class__.DICT_SECTION, key,  "[" + key + "]")


# =====================
#
# Handle config.ini
#
# =====================
class ConfigIni( Property ):
    HOME = str(Path.home())
    CONFIG_FOLDER = '.akoteka'
    INI_FILE_NAME="config.ini"

    # (section, key, default)
    DEFAULT_LANGUAGE = ("general", "language", "hu")
    DEFAULT_MEDIA_PATH = ("media", "media-path", ".")
    DEFAULT_MEDIA_PLAYER_VIDEO = ("media", "player-video", "mplayer")
    DEFAULT_MEDIA_PLAYER_VIDEO_PARAM = ("media", "player-video-param", "-zoom -fs -framedrop")
    DEFAULT_MEDIA_PLAYER_VIDEO_EXT = ("media", "player-video-ext", "divx,mkv,avi,mp4")
    DEFAULT_MEDIA_PLAYER_AUDIO = ("media", "player-audio", "rhythmbox")
    DEFAULT_MEDIA_PLAYER_AUDIO_PARAM = ("media", "player-audio-param", "")
    DEFAULT_MEDIA_PLAYER_AUDIO_EXT = ("media", "player-audio-ext", "mp3,ogg")
    
    
    __instance = None    

    def __new__(cls):
        if cls.__instance == None:
            cls.__instance = super().__new__(cls)
        return cls.__instance

    @classmethod
    def get_instance(cls):
        inst = cls.__new__(cls)
        cls.__init__(cls.__instance)
        return inst
        
    def __init__(self):
        folder = os.path.join(ConfigIni.HOME, ConfigIni.CONFIG_FOLDER)
        file = os.path.join(folder, ConfigIni.INI_FILE_NAME)
        super().__init__( file, True, folder )

    def get_language(self):
        return self.get(self.DEFAULT_LANGUAGE[0], self.DEFAULT_LANGUAGE[1], self.DEFAULT_LANGUAGE[2])

    def get_media_path(self):
        return self.get(self.DEFAULT_MEDIA_PATH[0], self.DEFAULT_MEDIA_PATH[1], self.DEFAULT_MEDIA_PATH[2])

    def get_media_player_video(self):
        return self.get(self.DEFAULT_MEDIA_PLAYER_VIDEO[0], self.DEFAULT_MEDIA_PLAYER_VIDEO[1], self.DEFAULT_MEDIA_PLAYER_VIDEO[2])

    def get_media_player_video_param(self):
        return self.get(self.DEFAULT_MEDIA_PLAYER_VIDEO_PARAM[0], self.DEFAULT_MEDIA_PLAYER_VIDEO_PARAM[1], self.DEFAULT_MEDIA_PLAYER_VIDEO_PARAM[2])

    def get_media_player_video_ext(self):
        return self.get(self.DEFAULT_MEDIA_PLAYER_VIDEO_EXT[0], self.DEFAULT_MEDIA_PLAYER_VIDEO_EXT[1], self.DEFAULT_MEDIA_PLAYER_VIDEO_EXT[2])

    def get_media_player_audio(self):
        return self.get(self.DEFAULT_MEDIA_PLAYER_AUDIO[0], self.DEFAULT_MEDIA_PLAYER_AUDIO[1], self.DEFAULT_MEDIA_PLAYER_AUDIO[2])

    def get_media_player_audio_param(self):
        return self.get(self.DEFAULT_MEDIA_PLAYER_AUDIO_PARAM[0], self.DEFAULT_MEDIA_PLAYER_AUDIO_PARAM[1], self.DEFAULT_MEDIA_PLAYER_AUDIO_PARAM[2])

    def get_media_player_audio_ext(self):
        return self.get(self.DEFAULT_MEDIA_PLAYER_AUDIO_EXT[0], self.DEFAULT_MEDIA_PLAYER_AUDIO_EXT[1], self.DEFAULT_MEDIA_PLAYER_AUDIO_EXT[2])


    def set_language(self, lang):
        self.update(self.DEFAULT_LANGUAGE[0], self.DEFAULT_LANGUAGE[1], lang)

    def set_media_path(self, path):
        self.update(self.DEFAULT_MEDIA_PATH[0], self.DEFAULT_MEDIA_PATH[1], path)

    def set_media_player_video(self, player):
        self.update(self.DEFAULT_MEDIA_PLAYER_VIDEO[0], self.DEFAULT_MEDIA_PLAYER_VIDEO[1], player)

    def set_media_player_video_param(self, param):
        self.update(self.DEFAULT_MEDIA_PLAYER_VIDEO_PARAM[0], self.DEFAULT_MEDIA_PLAYER_VIDEO_PARAM[1], param)

    def set_media_player_video_ext(self, param):
        self.update(self.DEFAULT_MEDIA_PLAYER_VIDEO_EXT[0], self.DEFAULT_MEDIA_PLAYER_VIDEO_EXT[1], param)

    def set_media_player_audio(self, player):
        self.update(self.DEFAULT_MEDIA_PLAYER_AUDIO[0], self.DEFAULT_MEDIA_PLAYER_AUDIO[1], player)

    def set_media_player_audio_param(self, param):
        self.update(self.DEFAULT_MEDIA_PLAYER_AUDIO_PARAM[0], self.DEFAULT_MEDIA_PLAYER_AUDIO_PARAM[1], param)

    def set_media_player_audio_ext(self, param):
        self.update(self.DEFAULT_MEDIA_PLAYER_AUDIO_EXT[0], self.DEFAULT_MEDIA_PLAYER_AUDIO_EXT[1], param)


config_ini = {}

def get_config_ini():
    return ConfigIni.get_instance()

def re_read_config_ini():
    global config_ini
    global dic
    
    ci = get_config_ini()
    
    # Read config.ini    
    config_ini['language'] = ci.get_language()
    config_ini['media_path'] = ci.get_media_path()
    config_ini['media_player_video'] = ci.get_media_player_video()
    config_ini['media_player_video_param'] = ci.get_media_player_video_param()
    config_ini['media_player_video_ext'] = ci.get_media_player_video_ext()
    config_ini['media_player_audio'] = ci.get_media_player_audio()
    config_ini['media_player_audio_param'] = ci.get_media_player_audio_param()
    config_ini['media_player_audio_ext'] = ci.get_media_player_audio_ext()

    # Get the dictionary
    dic = Dict.get_instance( config_ini['language'] )
    
re_read_config_ini()

# --------------------------------------------------------
# --------------------------------------------------------
#
# Gives back the translation of the word
#
# word      word which should be translated
#
# --------------------------------------------------------
# --------------------------------------------------------
def _(word):
    return dic._(word)



import os
import sys
#import json
import re
import configparser
import cgi, cgitb

from PyQt5.QtCore import QPoint
from PyQt5.QtCore import QRect
from PyQt5.QtCore import QSize
from PyQt5.QtCore import Qt

from PyQt5.QtWidgets import QApplication
from PyQt5.QtWidgets import QLayout
from PyQt5.QtWidgets import QPushButton
from PyQt5.QtWidgets import QSizePolicy
from PyQt5.QtWidgets import QWidget

from akoteka.handle_property import config_ini

filter_key = {
    "best":{
        "store-mode": "v",
        "key-dict-prefix": "title_",
        "value-dict": False,
        "section": "rating"
    },
    "new":{
        "store-mode": "v",
        "key-dict-prefix": "title_",
        "value-dict": False,
        "section": "rating"
    },
    "favorite":{
        "store-mode": "v",
        "key-dict-prefix": "title_",
        "value-dict": False,
        "section": "rating"
    },
    "director":{
        "store-mode": "a",
        "key-dict-prefix": "title_",
        "value-dict": False,
        "section": "general"
    },
    "actor": {
        "store-mode": "a",
        "key-dict-prefix": "title_",
        "value-dict": False,
        "section": "general"
    },
    "theme":{
        "store-mode": "a",
        "key-dict-prefix": "title_",
        "value-dict": True,
        "value-dict-prefix": "theme_",
        "section": "general"
    },
    "genre":{
        "store-mode": "a",
        "key-dict-prefix": "title_",
        "value-dict": True,
        "value-dict-prefix": "genre_",
        "section": "general"
    }
}


class FlowLayout(QLayout):

    def __init__(self, parent=None, margin=0, spacing=-1):
        super(FlowLayout, self).__init__(parent)

        if parent is not None:
            self.setContentsMargins(margin, margin, margin, margin)

        self.setSpacing(spacing)

        self.itemList = []

    def __del__(self):
        item = self.takeAt(0)
        while item:
            item = self.takeAt(0)

    def addItem(self, item):
        self.itemList.append(item)

    def count(self):
        return len(self.itemList)

    # to keep it in style with the others..
    def rowCount(self):
        return self.count()

    def itemAt(self, index):
        if index >= 0 and index < len(self.itemList):
            return self.itemList[index]

        return None

    def takeAt(self, index):
        if index >= 0 and index < len(self.itemList):
            return self.itemList.pop(index)

        return None

    def expandingDirections(self):
        return Qt.Orientations(Qt.Orientation(0))

    def hasHeightForWidth(self):
        return True

    def heightForWidth(self, width):
        height = self.doLayout(QRect(0, 0, width, 0), True)
        return height

    def setGeometry(self, rect):
        super(FlowLayout, self).setGeometry(rect)
        self.doLayout(rect, False)

    def sizeHint(self):
        return self.minimumSize()

    def minimumSize(self):
        size = QSize()

        for item in self.itemList:
            size = size.expandedTo(item.minimumSize())

        margin, _, _, _ = self.getContentsMargins()

        size += QSize(2 * margin, 2 * margin)
        return size

    def doLayout(self, rect, testOnly):
        x = rect.x()
        y = rect.y()
        lineHeight = 0

        for item in self.itemList:
            wid = item.widget()
            spaceX = self.spacing() + wid.style().layoutSpacing(QSizePolicy.PushButton, QSizePolicy.PushButton, Qt.Horizontal)
            spaceY = self.spacing() + wid.style().layoutSpacing(QSizePolicy.PushButton, QSizePolicy.PushButton, Qt.Vertical)
            nextX = x + item.sizeHint().width() + spaceX
            if nextX - spaceX > rect.right() and lineHeight > 0:
                x = rect.x()
                y = y + lineHeight + spaceY
                nextX = x + item.sizeHint().width() + spaceX
                lineHeight = 0

            if not testOnly:
                item.setGeometry(QRect(QPoint(x, y), item.sizeHint()))

            x = nextX
            lineHeight = max(lineHeight, item.sizeHint().height())

        return y + lineHeight - rect.y()
    

def get_pattern_video():
    ptrn = '|'.join( config_ini['media_player_video_ext'].split(",") )
    return re.compile( '^.+[.](' + ptrn + ')$' )    

def get_pattern_audio():
    ptrn = '|'.join( config_ini['media_player_audio_ext'].split(",") )
    return re.compile( '^.+[.](' + ptrn + ')$' )    

def get_pattern_image():
    return re.compile( '^image[.]jp(eg|g)$' )
    
def get_pattern_card():
    return re.compile('^card.ini$')

def folder_investigation( actual_dir, json_list):
    
    # Collect files and and dirs in the current directory
    file_list = [f for f in os.listdir(actual_dir) if os.path.isfile(os.path.join(actual_dir, f))] if os.path.exists(actual_dir) else []
    dir_list = [d for d in os.listdir(actual_dir) if os.path.isdir(os.path.join(actual_dir, d))] if os.path.exists(actual_dir) else []

    # now I got to a certain level of directory structure
    card_path_os = None
    media_path_os = None
    image_path_os = None
    media_name = None
    
    is_card_dir = True

    # Go through all files in the folder
    for file_name in file_list:

        #
        # collect the files which count
        #
        
        # find the Card
        if get_pattern_card().match( file_name ):
            card_path_os = os.path.join(actual_dir, file_name)
            
        # find the Media (video or audio)
        if get_pattern_audio().match(file_name) or get_pattern_video().match(file_name):
            media_path_os = os.path.join(actual_dir, file_name)
            media_name = file_name
            
        # find the Image
        if get_pattern_image().match( file_name ):
           image_path_os = os.path.join(actual_dir, file_name)


    card = {}
    
    title_json_list = {}
    title_json_list['hu'] = media_name
    title_json_list['en'] = media_name
    card['title'] = title_json_list

    storyline_json_list = {}
    storyline_json_list['hu'] = ""
    storyline_json_list['en'] = ""
    card['storyline'] = storyline_json_list
                
    general_json_list = {}
    general_json_list['year'] = ""
    general_json_list['director'] = [] #json.loads('[]')
    general_json_list['length'] = ""
    general_json_list['sound'] = [] #json.loads('[]')
    general_json_list['sub'] = [] #json.loads('[]')
    general_json_list['genre'] = [] #json.loads('[]')
    general_json_list['theme'] = [] #json.loads('[]')
    general_json_list['actor'] = [] #json.loads('[]')
    general_json_list['country'] = [] #json.loads('[]')
    card['general'] = general_json_list
                
    rating_json_list = {}
    rating_json_list['best'] = ""
    rating_json_list['new'] = ""
    rating_json_list['favorite'] = ""
    card['rating'] = rating_json_list
                                        
    card['links'] = {}

    extra_json_list = {}    
    extra_json_list['image-path'] = ""
    extra_json_list['media-path'] = ""
    extra_json_list['recent-folder'] = actual_dir
    extra_json_list['sub-cards'] = [] #json.loads('[]')
    card['extra'] = extra_json_list


    # ----------------------------------
    #
    # it is a COLLECTOR CARD dir
    #
    # there is:     -Card 
    #               -at least one Dir
    # ther is NO:   -Media
    #  
    # ----------------------------------
    if card_path_os and not media_path_os and dir_list:
                
        parser = configparser.RawConfigParser()
        parser.read(card_path_os)
        
        # I collect the data from the card and the image if there is and the folders if there are
        try:
            
            # save the http path of the image
            card['extra']['image-path'] = image_path_os

            # saves the os path of the media - There is no
            card['extra']['media-path'] = None
            
            card['title']['hu'] = parser.get("titles", "title_hu")
            card['title']['en'] = parser.get("titles", "title_en")
                
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            print(nop_err, "in ", card_path_os)
                # TODO It could be more sophisticated, depending what field failed

        json_list.append(card)
        
    # --------------------------------
    #
    # it is a MEDIA CARD dir
    #
    # there is:     -Card 
    #               -Media
    # 
    # --------------------------------
    elif card_path_os and media_path_os:

        # first collect every data from the card
        parser = configparser.RawConfigParser()
        parser.read(card_path_os)

        # save the os path of the image
        card['extra']['image-path'] = image_path_os            

        # saves the os path of the media
        card['extra']['media-path'] = media_path_os

        try:
            card['title']['hu'] = parser.get("titles", "title_hu")
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)

        try:            
            card['title']['en'] = parser.get("titles", "title_en")
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)

        try:
            card['storyline']['hu'] = parser.get("storyline", "storyline_hu")
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)

        try:
            card['storyline']['en'] = parser.get("storyline", "storyline_en")
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)

        try:            
            card['general']['year'] = parser.get("general", "year")
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)

        try:
            directors = parser.get("general", "director").split(",")            
            for director in directors:
                card['general']['director'].append(director.strip())
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)

        try:
            card['general']['length'] = parser.get("general", "length")
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)

        try:
            sounds = parser.get("general", "sound").split(",")
            for sound in sounds:
                card['general']['sound'].append(sound.strip())
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)

        try:
            subs = parser.get("general", "sub").split(",")
            for sub in subs:
                card['general']['sub'].append(sub.strip())
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)

        try:
            genres = parser.get("general", "genre").split(",")
            for genre in genres:
                card['general']['genre'].append(genre.strip())
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)

        try:
            themes = parser.get("general", "theme").split(",")
            for theme in themes:
                card['general']['theme'].append(theme.strip())
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)

        try:
            actors = parser.get("general", "actor").split(",")
            for actor in actors:
                card['general']['actor'].append(actor.strip())
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)

        try:
            countries = parser.get("general", "country").split(",")
            for country in countries:
                card['general']['country'].append(country.strip())
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)

        try:
            card['rating']['best'] = parser.get("rating", "best")
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)
            
        try:
            card['rating']['new'] = parser.get("rating", "new")
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)

        try:
            card['rating']['favorite'] = parser.get("rating", "favorite")
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)

        try:                                                
            card['links']['imdb'] = parser.get("links", "imdb")
        except (configparser.NoSectionError, configparser.NoOptionError) as nop_err:
            log_msg(nop_err, "in ", card_path_os)

        json_list.append(card)

    # ----------------------------------
    #
    # it is NO CARD dir
    #
    # ----------------------------------
    else:
        
        is_card_dir = False
        
    # ----------------------------
    #
    # Through the Sub-directories
    #
    # ----------------------------    
    for name in dir_list:
        subfolder_path_os = os.path.join(actual_dir, name)
        folder_investigation( subfolder_path_os, card['extra']['sub-cards'] if is_card_dir else json_list )

    # and finaly returns
    return

def clearLayout(layout):
    while layout.count():
        child = layout.takeAt(0)
        if child.widget():
            child.widget().deleteLater()
 

def log_msg(par1, par2, par3):
    print(par1, par2, par3)
    

def collect_cards( rootdirs ):    
    media_list = [] #json.loads('[]')

    for rootdir in rootdirs:
        folder_investigation(rootdir, media_list)

    return media_list



import sys
import os

from pkg_resources import resource_string, resource_filename

from akoteka.gui.pyqt_import import *

from akoteka.constants import *
from akoteka.handle_property import _
from akoteka.handle_property import config_ini

# ====================
#
# Configuration Dialog
#
# ====================
class ConfigurationDialog(QDialog):

    def __init__(self):
        super().__init__()    
    
        self.setWindowTitle(_('title_settings'))
        self.resize(WINDOW_WIDTH, WINDOW_HEIGHT)

        self.self_layout = QVBoxLayout(self)
        self.self_layout.setSpacing(0)
        self.setLayout(self.self_layout)
        
        self.content_section = ContentSection()
        
        self.self_layout.addWidget(self.content_section)
                
        self.self_layout.addStretch(1)
        
        button_box_section = ButtonBoxSection(self)
        self.self_layout.addWidget(button_box_section)
        
        # language
        self.language_selector = LanguageSelector(config_ini['language'])
        self.content_section.addWidget(self.language_selector)
        
        # media path
        self.media_path_selector = MediaPathSelector(config_ini['media_path'])
        self.content_section.addWidget(self.media_path_selector)
        
        self.content_section.addWidget(QHLine())

        # video player 
        self.media_player_video_selector = MediaPlayerVideoSelector(config_ini['media_player_video'])
        self.content_section.addWidget(self.media_player_video_selector)
        
        # video player parameters        
        self.media_player_video_param = MediaPlayerVideoParam(config_ini['media_player_video_param'])
        self.content_section.addWidget(self.media_player_video_param)

        self.content_section.addWidget(QHLine())

        # audio player 
        self.media_player_audio_selector = MediaPlayerAudioSelector(config_ini['media_player_audio'])
        self.content_section.addWidget(self.media_player_audio_selector)
        
        # audio player parameters        
        self.media_player_audio_param = MediaPlayerAudioParam(config_ini['media_player_audio_param'])
        self.content_section.addWidget(self.media_player_audio_param)
    
    def get_media_path(self):
        return self.media_path_selector.get_media_path()
        
    def get_language(self):
        return self.language_selector.get_language()
 
    def get_media_player_video(self):
        return self.media_player_video_selector.get_media_player_video() 
    
    def get_media_player_video_param(self):
        return self.media_player_video_param.get_media_player_video_param()

    def get_media_player_audio(self):
        return self.media_player_audio_selector.get_media_player_audio() 
    
    def get_media_player_audio_param(self):
        return self.media_player_audio_param.get_media_player_audio_param()

class ContentSection(QWidget):
        def __init__(self):
            super().__init__()    
    
            self.self_layout = QVBoxLayout(self)
            self.self_layout.setSpacing(3)
            self.setLayout(self.self_layout)
        
        def addWidget(self, widget):
            self.self_layout.addWidget(widget)

class ButtonBoxSection(QWidget):
        def __init__(self, parent):
            super().__init__()    
    
            self.self_layout = QHBoxLayout(self)
            self.setLayout(self.self_layout)
            
            button_box = QDialogButtonBox(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
            self.self_layout.addWidget( button_box )

            button_box.button(QDialogButtonBox.Ok).setText(_('button_ok'))
            button_box.button(QDialogButtonBox.Cancel).setText(_('button_cancel'))
            button_box.accepted.connect(parent.accept)
            button_box.rejected.connect(parent.reject)
            

class LineTemplate(QWidget):
    def __init__(self, label):
        super().__init__()    
    
        self.self_layout=QHBoxLayout(self)
        self.self_layout.setContentsMargins(0,0,0,0)
        self.setStyleSheet("border:1px") 
        self.setLayout(self.self_layout)

        # Label
        label = QLabel(label)
        label.setMinimumWidth(LABEL_LENGTH)
        label.setMaximumWidth(LABEL_LENGTH)
        self.self_layout.addWidget(label)
        
# ====================
#
# Language Selector
#
# ====================
class LanguageSelector(LineTemplate):
    
    def __init__(self, default_language):
        super().__init__(_('title_language') + ':')
        
        self.language_combo = QComboBox()
        self.language_combo.addItem(_('lang_hu'), 'hu')
        self.language_combo.addItem(_('lang_en'), 'en')
        style_box = "QComboBox { max-width: " + LANGUAGE_DROPDOWN_LENGTH + "px; min-width: " + LANGUAGE_DROPDOWN_LENGTH + "px; border: 1px solid gray; border-radius: 5px; } QComboBox QAbstractItemView {color: black; border: 1px solid gray}"

        # drop-down menu         
        self.language_combo.setStyleSheet(style_box)
        self.self_layout.addStretch(1)
        self.self_layout.addWidget(self.language_combo)
        
        # empty space at the end
        self.empty_button = QPushButton()
        self.empty_button.setCheckable(False)
        empty_icon = QIcon()
        empty_icon.addPixmap(QPixmap( resource_filename(__name__,os.path.join("img", IMG_EMPTY_BUTTON)) ), QIcon.Normal, QIcon.On)
        self.empty_button.setIcon( empty_icon )
        self.empty_button.setIconSize(QSize(IMG_SIZE,IMG_SIZE))
        self.self_layout.addWidget(self.empty_button)
        
        # select the language from config.ini file
        self.language_combo.setCurrentIndex( self.language_combo.findData(default_language) )
        
    def get_language(self):
        return self.language_combo.itemData( self.language_combo.currentIndex() )

# ====================
#
# Media Path Selector
#
# ====================
class MediaPathSelector(LineTemplate):
    
    def __init__(self, default_media_path):
        super().__init__(_('title_media_path') + ':')    

        # Text Field
        self.folder_field = QLineEdit(self)
        self.folder_field.setText(default_media_path)
        self.self_layout.addWidget(self.folder_field)
        
        # Button
        self.folder_selector_button = QPushButton()
        self.folder_selector_button.setCheckable(False)
        selector_icon = QIcon()
        selector_icon.addPixmap(QPixmap( resource_filename(__name__,os.path.join("img", IMG_FOLDER_BUTTON)) ), QIcon.Normal, QIcon.On)
        self.folder_selector_button.setIcon( selector_icon )
        self.folder_selector_button.setIconSize(QSize(IMG_SIZE,IMG_SIZE))
        self.folder_selector_button.setCursor(QCursor(Qt.PointingHandCursor))
        self.folder_selector_button.setStyleSheet("background:transparent; border:none") 
        self.folder_selector_button.clicked.connect(self.select_folder_button_on_click)

        self.self_layout.addWidget(self.folder_selector_button)

    def get_media_path(self):
        return self.folder_field.text()
    
    def select_folder_button_on_click(self):
        
#folder = QFileDialog.getExistingDirectory(self, _('title_select_media_path'), glob.media_path, QFileDialog.ShowDirsOnly)
        folder = QFileDialog.getExistingDirectory(self, _('title_select_media_path'), config_ini['media_path'], QFileDialog.ShowDirsOnly)
        
        if folder:
            self.folder_field.setText(folder)
        
# ====================
#
# Media Player Video
#
# ====================
class MediaPlayerVideoSelector(LineTemplate):
    
    def __init__(self, default_media_player_video_path):
        super().__init__(_('title_media_player_video') + ':')

        # Text Field
        self.file_field = QLineEdit(self)
        self.file_field.setText(default_media_player_video_path)
        self.self_layout.addWidget(self.file_field)
        
        # Button
        self.folder_selector_button = QPushButton()
        self.folder_selector_button.setCheckable(False)
        selector_icon = QIcon()
        selector_icon.addPixmap(QPixmap( resource_filename(__name__,os.path.join("img", IMG_FOLDER_BUTTON)) ), QIcon.Normal, QIcon.On)
        self.folder_selector_button.setIcon( selector_icon )
        self.folder_selector_button.setIconSize(QSize(IMG_SIZE,IMG_SIZE))
        self.folder_selector_button.setCursor(QCursor(Qt.PointingHandCursor))
        self.folder_selector_button.setStyleSheet("background:transparent; border:none") 
        self.folder_selector_button.clicked.connect(self.select_folder_button_on_click)

        self.self_layout.addWidget(self.folder_selector_button)

    def get_media_player_video(self):
        return self.file_field.text()
    
    def select_folder_button_on_click(self):        
        file, valami = QFileDialog.getOpenFileName(self, _('title_select_media_player_video'), os.path.abspath(os.sep))
        
        if file:
            self.file_field.setText(file)


# ========================
#
# Media Player Video Param
#
# ========================
class MediaPlayerVideoParam(LineTemplate):
    
    def __init__(self, default_media_player_video_param):
        super().__init__( _('title_media_player_video_param') + ':' )
    
        # Text Field
        self.folder_field = QLineEdit(self)
        self.folder_field.setText(default_media_player_video_param)
        self.self_layout.addWidget(self.folder_field)
        
        self.empty_button = QPushButton()
        self.empty_button.setCheckable(False)
        empty_icon = QIcon()
        empty_icon.addPixmap(QPixmap( resource_filename(__name__,os.path.join("img", IMG_EMPTY_BUTTON)) ), QIcon.Normal, QIcon.On)
        self.empty_button.setIcon( empty_icon )
        self.empty_button.setIconSize(QSize(IMG_SIZE,IMG_SIZE))
        self.self_layout.addWidget(self.empty_button)

        
    def get_media_player_video_param(self):
        return self.folder_field.text()
    
# ====================
#
# Media Player Audio
#
# ====================
class MediaPlayerAudioSelector(LineTemplate):
    
    def __init__(self, default_media_player_audio_path):
        super().__init__(_('title_media_player_audio') + ':')

        # Text Field
        self.file_field = QLineEdit(self)
        self.file_field.setText(default_media_player_audio_path)
        self.self_layout.addWidget(self.file_field)
        
        # Button
        self.folder_selector_button = QPushButton()
        self.folder_selector_button.setCheckable(False)
        selector_icon = QIcon()
        selector_icon.addPixmap(QPixmap( resource_filename(__name__,os.path.join("img", IMG_FOLDER_BUTTON)) ), QIcon.Normal, QIcon.On)
        self.folder_selector_button.setIcon( selector_icon )
        self.folder_selector_button.setIconSize(QSize(IMG_SIZE,IMG_SIZE))
        self.folder_selector_button.setCursor(QCursor(Qt.PointingHandCursor))
        self.folder_selector_button.setStyleSheet("background:transparent; border:none") 
        self.folder_selector_button.clicked.connect(self.select_folder_button_on_click)

        self.self_layout.addWidget(self.folder_selector_button)

    def get_media_player_audio(self):
        return self.file_field.text()
    
    def select_folder_button_on_click(self):        
        file, valami = QFileDialog.getOpenFileName(self, _('title_select_media_player_audio'), os.path.abspath(os.sep))
        
        if file:
            self.file_field.setText(file)


# ========================
#
# Media Player Audio Param
#
# ========================
class MediaPlayerAudioParam(LineTemplate):
    
    def __init__(self, default_media_player_audio_param):
        super().__init__( _('title_media_player_audio_param') + ':' )
    
        # Text Field
        self.folder_field = QLineEdit(self)
        self.folder_field.setText(default_media_player_audio_param)
        self.self_layout.addWidget(self.folder_field)
        
        self.empty_button = QPushButton()
        self.empty_button.setCheckable(False)
        empty_icon = QIcon()
        empty_icon.addPixmap(QPixmap( resource_filename(__name__,os.path.join("img", IMG_EMPTY_BUTTON)) ), QIcon.Normal, QIcon.On)
        self.empty_button.setIcon( empty_icon )
        self.empty_button.setIconSize(QSize(IMG_SIZE,IMG_SIZE))
        self.self_layout.addWidget(self.empty_button)

        
    def get_media_player_audio_param(self):
        return self.folder_field.text()

    
    
    



import sys
import math
import time 
import os
from itertools import cycle
from datetime import datetime

from PyQt5.QtWidgets import QWidget
from PyQt5.QtWidgets import QVBoxLayout
from PyQt5.QtWidgets import QHBoxLayout
from PyQt5.QtWidgets import QScrollArea
from PyQt5.QtWidgets import QDesktopWidget
from PyQt5.QtWidgets import QPushButton
from PyQt5.QtWidgets import QLabel
from PyQt5.QtWidgets import QFrame
from PyQt5.QtWidgets import QSpacerItem
from PyQt5.QtWidgets import QSizePolicy
from PyQt5.QtWidgets import QApplication
from PyQt5.QtWidgets import QComboBox
from PyQt5.QtWidgets import QCheckBox
from PyQt5.QtWidgets import QStyleFactory
from PyQt5.QtWidgets import QFileDialog
from PyQt5.QtWidgets import QInputDialog
from PyQt5.QtWidgets import QDialog
from PyQt5.QtWidgets import QDialogButtonBox
from PyQt5.QtWidgets import QLineEdit


from PyQt5.QtGui import QFont
from PyQt5.QtGui import QColor
from PyQt5.QtGui import QPalette
from PyQt5.QtGui import QPixmap
from PyQt5.QtGui import QPainter
from PyQt5.QtGui import QIcon
from PyQt5.QtGui import QCursor
from PyQt5.QtGui import QDrag
from PyQt5.QtGui import QMovie
from PyQt5.QtGui import QPen

from PyQt5 import QtCore

from PyQt5.QtCore import Qt
from PyQt5.QtCore import QSize
from PyQt5.QtCore import QAbstractTableModel
from PyQt5.QtCore import QModelIndex
from PyQt5.QtCore import QVariant
from PyQt5.QtCore import QThread
from PyQt5.QtCore import pyqtSignal
from PyQt5.QtCore import QMimeData
from PyQt5.QtCore import QPoint
from PyQt5.QtCore import QByteArray

from pkg_resources import resource_string, resource_filename
 
# =========================
#
# Card Holder
#
# =========================
class CardHolder( QWidget ):
    
    resized = QtCore.pyqtSignal(int,int)
    moved_to_front = QtCore.pyqtSignal(int)
    
    DEFAULT_SPINNER_NAME = 'spinner.gif'

    DEFAULT_MAX_OVERLAPPED_CARDS = 4    
    DEFAULT_BORDER_WIDTH = 5
    DEFAULT_BACKGROUND_COLOR = QColor(Qt.red)
    DEFAULT_BORDER_RADIUS = 10
    
    DEFAULT_RATE_OF_CARD_WIDTH_DECLINE = 10
    DEFAULT_RATE_OF_CARD_Y_MULTIPLICATOR = 6
    
    CARD_TRESHOLD = 6
    MAX_CARD_ROLLING_RATE = 10
    
    def __init__(self, 
                 parent, 
                 get_new_card_method, 
                 get_collected_cards_method,
                 collecting_start_method = None,
                 collecting_finish_method = None,
                 ):
        QWidget.__init__(self, parent)

        self.parent = parent
        self.get_new_card_method = get_new_card_method
        self.get_collected_cards_method = get_collected_cards_method
        self.collecting_start_method = collecting_start_method
        self.collecting_finish_method = collecting_finish_method
        
        self.shown_card_list = []
        self.card_descriptor_list = []

        self.set_max_overlapped_cards(CardHolder.DEFAULT_MAX_OVERLAPPED_CARDS, False)        
        self.set_border_width(CardHolder.DEFAULT_BORDER_WIDTH, False)
        self.set_border_radius(CardHolder.DEFAULT_BORDER_RADIUS, False)
        self.set_background_color(CardHolder.DEFAULT_BACKGROUND_COLOR, False)
        
        # -----------
        # self_layout
        # -----------
        self.self_layout = QVBoxLayout(self)
        self.setLayout(self.self_layout)

        self.actual_card_index = 0
        self.rate_of_movement = 0
        self.delta_rate = 0
        
        self.countDown = CountDown()
        self.countDown.timeOver.connect(self.animated_move_to_closest_descreet_position)
        
        self.collecting_spinner = None
        spinner_file_name = resource_filename(__name__,os.path.join("img", CardHolder.DEFAULT_SPINNER_NAME))
        self.setup_spinner(spinner_file_name)

        self.set_y_coordinate_by_reverse_index_method(self.get_y_coordinate_by_reverse_index)
        self.set_x_offset_by_index_method(self.get_x_offset_by_index)

        self.mouse_already_pressed = False

        # The CardHolder will accept Focus by Tabbing and Clicking
        self.setFocusPolicy(Qt.StrongFocus)

        # it hides the CardHolder until it is filled up with cards
        self.select_index(0)
        
    def set_y_coordinate_by_reverse_index_method(self, method):
        self.get_y_coordinate_by_reverse_index_method = method
        
    def get_y_coordinate_by_reverse_index(self, reverse_index, diff_to_max):
        
        raw_pos = (reverse_index + diff_to_max) * (reverse_index + diff_to_max)
        offset = diff_to_max * diff_to_max
        return (raw_pos - offset) * CardHolder.DEFAULT_RATE_OF_CARD_Y_MULTIPLICATOR

    def set_x_offset_by_index_method(self, method):
        self.get_x_offset_by_index_method = method

    def get_x_offset_by_index(self, index):
        return index * CardHolder.DEFAULT_RATE_OF_CARD_WIDTH_DECLINE

    # ----------------
    #
    # setup Spinner
    #
    # ----------------
    def setup_spinner(self, file_name):

        # remove the old spinner
        if self.collecting_spinner != None:
            self.spinner_movie.stop()            
            self.collecting_spinner.setParent(None)
            self.collecting_spinner = None
            
        self.spinner_movie = QMovie(file_name, QByteArray(), self)
        self.collecting_spinner = QLabel(self.parent)
        self.collecting_spinner.setMovie(self.spinner_movie)
        self.collecting_spinner.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

        self.spinner_movie.setCacheMode(QMovie.CacheAll)
        self.spinner_movie.setSpeed(100)
        self.spinner_movie.start()          # if I do not start it, it stays hidden later
        self.spinner_movie.stop()
        self.collecting_spinner.move(0,0)
        self.collecting_spinner.show()
        self.collecting_spinner.setHidden(True)

        img_size = self.spinner_movie.currentPixmap().size()
        self.collecting_spinner.resize(img_size.width(), img_size.height())      










    # ---------------------------------------------------------------------
    # ---------------------------------------------------------------------
    #
    # start card collection
    #
    # ---------------------------------------------------------------------
    # ---------------------------------------------------------------------
    def startCardCollection(self, parameters):
        """
        This method should be called from outside when you want a totally new collection of cards
        Usualy that happens only once, at the beginning.
        The process could take lot of time, so a Spinner will indicate that the loding is in-process
        """

        self.cc = CollectCardsThread.get_instance( self, self.get_collected_cards_method, parameters )
        if self.cc:
            self.cc.collectionStarted.connect(self.card_collection_started)
            self.cc.collectionFinished.connect(self.card_collection_finished)
            self.cc.start()   

    def card_collection_started(self):
        """
        This method is emitted by the CollectCardsThread when it starts
        """
        
        # Start the Spinner
        self.collecting_spinner.move(
            (self.parent.geometry().width()-self.collecting_spinner.width()) / 2,
            (self.parent.geometry().height()-self.collecting_spinner.width()) / 2
        )
        self.spinner_movie.start()
        self.collecting_spinner.setHidden(False)        

        # Remove everything from the CardHolder
        self.fillUpCardHolderByDescriptor([])
        
        # Inform the parent if it is necessary
        if self.collecting_start_method:
            self.collecting_start_method()
        

    def card_collection_finished(self, card_descriptor_structure):
        """
        This method is emitted by the CollectCardsThread when it finishes the
        collection of the cards descriptor
        parameters:
                    card_descriptor_structure:  the collected descriptor without filtering
        """
        
        # Stop the Spinner
        self.collecting_spinner.setHidden(True)
        self.spinner_movie.stop()

        # Fill up the CardHolder with the new Cards
        self.fillUpCardHolderByDescriptor(card_descriptor_structure)

        # Inform the parent if it is necessary
        if self.collecting_finish_method:
           self.collecting_finish_method(self, card_descriptor_structure) 

    # ---------------------------------------------------------------------
    # ---------------------------------------------------------------------
    #
    # Fill-up Card Descriptor List
    #
    # ---------------------------------------------------------------------
    # ---------------------------------------------------------------------
    def fillUpCardHolderByDescriptor(self, card_descriptor_list):
        """
        This method fill up the card_descriptor_structure and then fill up the CardHolder
        with the new Cards
        """
       
        # Fill up the card_descriptor_structure
        self.card_descriptor_list = []
        for c in card_descriptor_list:
            self.card_descriptor_list.append(c)
        
        # Build up the CardHolder and select the actual card
        self.select_actual_card()







        
        
    def set_border_width(self, width, update=True):
        self.border_width = width
        if update:
            self.update()
        
    def get_border_width(self):
        return self.border_width

    def set_background_color(self, color, update=True):
        self.background_color = color
        ## without this line it wont paint the background, but the children get the background color info
        ## with this line, the rounded corners will be ruined
        #self.setAttribute(QtCore.Qt.WA_StyledBackground, True)
        #self.setStyleSheet('background-color: ' + self.background_color.name())
        if update:
            self.update()
        
    def set_max_overlapped_cards(self, number, update=True):
        self.max_overlapped_cards = number
        if update:
            self.select_index(self.actual_card_index)
        
    def set_border_radius(self, radius, update=True):
        self.border_radius = radius
        if update:
            self.update()
        
    def get_max_overlapped_cards(self):
        return self.max_overlapped_cards
  
    def resizeEvent(self, event):
        self.resized.emit(event.size().width(),event.size().height())
        return super(CardHolder, self).resizeEvent(event)
 
    def remove_all_cards(self):
        for card in self.shown_card_list:
            card.setParent(None)
            card = None

    def remove_card(self, card):
        card.setParent(None)
        card = None

    def get_rate_of_movement(self):
        return self.rate_of_movement
    
    def get_delta_rate(self):
        return self.delta_rate

      
    # --------------------------------------------------------------------------
    #
    # Select Actual Card
    #
    # --------------------------------------------------------------------------
    def select_actual_card(self):
        """
        It builds up from scrach the shown_card_list from the card_descriptor_structure
        In the 0. position will be the Card identified by the actual_card_index.
        The card in the 0. position will be indicated as the "selected"        
        """
        
        self.select_index(self.actual_card_index)
    
    # --------------------------------------------------------------------------
    #
    # Select Index
    #
    # --------------------------------------------------------------------------
    def select_index(self, index):
        """
        It builds up from scrach the shown_card_list from the card_descriptor_list
        In the 0. position will be the Card identified by the "index" parameter
        The card in the 0. position will be indicated as the "selected"
        """
        
        index_corr = self.index_correction(index)
        
        self.actual_card_index = index_corr
        self.remove_all_cards()
        position = None
        self.shown_card_list = [None for i in range(index_corr + min(self.max_overlapped_cards, len(self.card_descriptor_list)-1), index_corr - 1, -1) ]

        for i in range( index_corr + min(self.max_overlapped_cards, len(self.card_descriptor_list)-1), index_corr - 1, -1):
            
            i_corr = self.index_correction(i)
            
            if( i_corr < len(self.card_descriptor_list)):

                local_index = i-index_corr
                card = self.get_new_card_method(self.card_descriptor_list[i_corr], local_index, i_corr )
                position = card.place(local_index)
                
                self.shown_card_list[local_index] = card

        # if there is at least one Card
        if self.shown_card_list:

            # Control the Height of the CardHolder
            self.setMinimumHeight(position[0].y() + position[1].y() + self.border_width )
            self.setMaximumHeight(position[0].y() + position[1].y() + self.border_width )
        
            # for indicating the selected (0) Card
            self.rolling(0)

        # if ther is NO Card at all
        else:
            # hides the CardHolder
            self.setMinimumHeight(0)
            self.setMaximumHeight(0)

    # -----------------------------------
    # -----------------------------------
    #
    # Animated Move To The Next Card
    #
    # -----------------------------------
    # -----------------------------------    
    def animatedMoveToNextCard(self):
        """
        for some reason the 2nd parameter is
        False when the animated_move_to_next
        is connected and emited, so that is
        why this intermediary method is needed
        """
        self.animated_move_to_next()

    # -----------------------------------
    # -----------------------------------
    #
    # Animated Move To The Previous Card
    #
    # -----------------------------------
    # -----------------------------------
    def animatedMoveToPreviousCard(self):
        """
        for some reason the 2nd parameter is
        False when the animated_move_to_next
        is connected and emited, so that is
        why this intermediary method is needed
        """
        self.animated_move_to_previous()

    # ---------------------------------
    #
    # Animated shows the next card
    #
    # ---------------------------------
    def animated_move_to_next(self, sleep=0.01):
        rate = self.get_rate_of_movement()
        
        if rate > 0:
            loop = self.MAX_CARD_ROLLING_RATE - rate
        elif rate < 0:
            loop = -rate
        else:
            loop = self.MAX_CARD_ROLLING_RATE
            
        self.animate = AnimateRolling.get_instance(int(loop), 1, sleep)
        if self.animate:
            self.animate.positionChanged.connect(self.rolling)
            self.animate.start()
       
    # ---------------------------------
    #
    # Animated shows the previous card
    #
    # ---------------------------------
    def animated_move_to_previous(self, sleep=0.01):
        rate = self.get_rate_of_movement()
        
        if rate > 0:
            loop = rate
        elif rate < 0:
            loop = self.MAX_CARD_ROLLING_RATE + rate
        else:
            loop = self.MAX_CARD_ROLLING_RATE
            
        self.animate = AnimateRolling.get_instance(int(loop), -1, sleep)
        if self.animate:
            self.animate.positionChanged.connect(self.rolling)
            self.animate.start()
    
    # ------------------------------------------------
    #
    # Animated moves to the closest descreet position
    #
    # ------------------------------------------------
    def animated_move_to_closest_descreet_position(self):
        rate = self.get_rate_of_movement()
        delta_rate = self.get_delta_rate()
        
        if rate != 0:
            
            if rate > 0:
                
                if delta_rate < 0:
                    value = -1
                    loop = rate
                else:
                    value = 1
                    loop = self.MAX_CARD_ROLLING_RATE - rate                    
                
                #if rate >= self.CARD_TRESHOLD:
                #    value = 1
                #    loop = self.MAX_CARD_ROLLING_RATE - rate                    
                #else:
                #    value = -1
                #    loop = rate
        
            elif rate < 0:
                
                if delta_rate < 0:
                    value = -1
                    loop = self.MAX_CARD_ROLLING_RATE + rate
                else:
                    value = 1
                    loop = -rate
                
                #if rate <= -self.CARD_TRESHOLD:
                #    value = -1
                #    loop = self.MAX_CARD_ROLLING_RATE + rate
                #else:
                #    value = 1
                #    loop = -rate
                
            self.animate = AnimateRolling.get_instance(int(loop), value, 0.02)
            if self.animate:
                self.animate.positionChanged.connect(self.rolling)
                self.animate.start()


        #selected_card = self.shown_card_list[0]
        #print(selected_card)
        #cd = self.card_descriptor_list[0]
        #print(cd)

        #print(self.card_descriptor_list.index(self.shown_card_list[0]))
        


    # ------------------------------------------------
    #
    # Animated moves to the set position Card
    #
    # ------------------------------------------------
    def animated_move_to(self, position, sleep=0.03):
        self.animate = AnimateRolling.get_instance(position * self.MAX_CARD_ROLLING_RATE, 1, sleep)
        if self.animate:
            self.animate.positionChanged.connect(self.rolling)
            self.animate.start()
        
        
        

    # ---------------------------------------------------------------------
    #
    # Rolling Wheel
    #
    # Rolls the cards according to the self.rate_of_movement + delta_rate
    # Plus it starts a Timer. If the timer expires, the cards will be 
    # moved to the first next discreet position
    #
    # --------------------------------------------------------------------
    def rolling_wheel(self, delta_rate):
        self.rolling(delta_rate)
        
        if self.rate_of_movement != 0:
            self.countDown.start()

    # --------------------------------------------------------------------
    #
    # ROLLING
    #
    # Rolls the cards according to the self.rate_of_movement + delta_rate
    #
    # delta_rate:   In normal case it is +1 or -1 (1/10 of the move to the next position)
    #               Adding this value to the self.rate_of_movement, it
    #               shows that how far the cards moved negativ (up) or
    #               positive (down) direction compared to the default
    #               (local_index) value
    #               -10 means (-100%) it moved to the next position, 
    #               +10 means (+100%) it moved to the previous position
    #
    # -------------------------------------------------------------------
    def rolling(self, delta_rate):
        if not self.shown_card_list:
            return        
        
        self.delta_rate = delta_rate
        
        if self.rate_of_movement >= self.MAX_CARD_ROLLING_RATE or self.rate_of_movement <= -self.MAX_CARD_ROLLING_RATE:
            self.rate_of_movement = 0

        self.rate_of_movement = self.rate_of_movement + delta_rate

        # Did not start to roll
        # if number of the shown cards are != min(max overlapped cards +1, len(self.card_descriptor_list))
        #if len(self.shown_card_list) <= self.get_max_overlapped_cards() + 1:
        if len(self.shown_card_list) == min(self.max_overlapped_cards + 1, len(self.card_descriptor_list)):
            
            # indicates that the first card is not the selected anymore
            card = self.shown_card_list[0]
            card.set_not_selected()
            
            # add new card to the beginning
            first_card = self.shown_card_list[0]
            first_card_index = self.index_correction(first_card.index - 1)
            card = self.get_new_card_method(self.card_descriptor_list[first_card_index], -1, first_card_index ) 
            self.shown_card_list.insert(0, card)
            
            # add a new card to the end
            last_card = self.shown_card_list[len(self.shown_card_list)-1]                
            last_card_index = self.index_correction(last_card.index + 1)
            card = self.get_new_card_method(self.card_descriptor_list[last_card_index], min(self.max_overlapped_cards + 1, len(self.card_descriptor_list)), last_card_index ) 
            self.shown_card_list.append(card)
            
            # Re-print to avoid the wrong-overlapping
            for card in self.shown_card_list[::-1]:
                card.setParent(None)
                card.setParent(self)
                card.show()        

        # adjust
        rate = self.rate_of_movement / self.MAX_CARD_ROLLING_RATE
        if self.rate_of_movement >= 0:
            self.rolling_adjust_forward(rate)
        else:
            self.rolling_adjust_backward(rate)

        # indicates that the first card is the selected            
        if self.rate_of_movement == 0:
            card = self.shown_card_list[0]
            card.set_selected()

        # show the cards in the right position
        rate = self.rate_of_movement / self.MAX_CARD_ROLLING_RATE
        for i, card in enumerate(self.shown_card_list):
            virtual_index = card.local_index - rate
#            card.refresh_color()
            card.place(virtual_index, True)

        self.actual_card_index = self.shown_card_list[0].index

    def rolling_adjust_forward(self,rate):
        
        if rate >= 1.0:
            self.rate_of_movement = 0
            
            # remove the first 2 cards from the list and from CardHolder
            for i in range(2):
                card_to_remove = self.shown_card_list.pop(0)
                card_to_remove.setParent(None)

            # re-index
            for i, card in enumerate(self.shown_card_list):
                card.local_index = i
                
        elif rate == 0:
            # remove the first card from the list and from CardHolder
            card_to_remove = self.shown_card_list.pop(0)
            card_to_remove.setParent(None)
            
            # remove the last card from the list and from CardHolder
            card_to_remove = self.shown_card_list.pop(len(self.shown_card_list) - 1)
            card_to_remove.setParent(None)

    def rolling_adjust_backward(self,rate):
        
        if rate <= -1.0:
            self.rate_of_movement = 0
        
            # remove the 2 last cards from the list and from CardHolder
            for i in range(2):
                card_to_remove = self.shown_card_list.pop(len(self.shown_card_list) - 1)
                card_to_remove.setParent(None)
            
            # re-index
            for i, card in enumerate(self.shown_card_list):
                card.local_index = i

    # -----------------------------------------------------------
    #
    # INDEX CORRECTION
    #
    # if the index > numbers of the cards then it gives back 0
    #    as the next index
    # if the index < 0 then it gives back the index of the last
    #    card as the previous index
    #
    # that is how it actually accomplishes an endless loop
    #
    # -----------------------------------------------------------
    def index_correction(self, index):
        if self.card_descriptor_list:
            return (len(self.card_descriptor_list) - abs(index) if index < 0 else index) % len(self.card_descriptor_list)
        else:
            return index

    def paintEvent(self, event):
        s = self.size()
        qp = QPainter()
        qp.begin(self)
        qp.setRenderHint(QPainter.Antialiasing, True)
        qp.setBrush( self.background_color )

        qp.drawRoundedRect(0, 0, s.width(), s.height(), self.border_radius, self.border_radius)
        qp.end()  

    def wheelEvent(self, event):
        modifiers = QApplication.keyboardModifiers()
        value = event.angleDelta().y()/8/15   # in normal case it is +1 or -1
        self.rolling_wheel(value)
  
  
    # ------------------------------
    #
    # Key Press Event: Up-Down arrow
    #
    # ------------------------------
    def keyPressEvent(self, event):

        if event.key() == QtCore.Qt.Key_Up:
            self.animated_move_to_next(sleep=0.03)
        elif event.key() == QtCore.Qt.Key_Down:
            self.animated_move_to_previous(sleep=0.03)
        event.ignore()
  
        
    # --------------
    #
    # MOUSE handling
    #
    # --------------        
    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.mouse_already_pressed = True
            self.drag_start_position = event.pos()
        event.ignore()

    def mouseMoveEvent(self, event):
        if event.buttons() == Qt.LeftButton and self.mouse_already_pressed:

            # Rolling Cards
            delta_y = event.pos().y() - self.drag_start_position.y()
            self.drag_start_position = event.pos()

            if delta_y > 0:
                self.rolling_wheel(1)
            elif delta_y < 0:
                self.rolling_wheel(-1)
        event.ignore()
               
    def mouseReleaseEvent(self, event):
        if event.button() == Qt.LeftButton and self.mouse_already_pressed:
            self.mouse_already_pressed = False        
        event.ignore()
               

  
  
  
# ==========================================================
#
# Panel
#
# This Panel is inside the Card and contains all widget
# what the user what to show on a Card.
# This Panel has rounded corner which is calculated by the
# radius of the Card and the widht of the border.
#
# ==========================================================
class Panel(QWidget):
    DEFAULT_BORDER_WIDTH = 5
    DEFAULT_BACKGROUND_COLOR = QColor(Qt.lightGray)
    
    def __init__(self, parent):
        #super().__init__()
        QWidget.__init__(self, parent)
        
        self.self_layout = QVBoxLayout()
        self.self_layout.setSpacing(1)
        self.setLayout(self.self_layout)

        self.set_background_color(Panel.DEFAULT_BACKGROUND_COLOR, False)        
        self.set_border_width(Panel.DEFAULT_BORDER_WIDTH, False)
        #self.set_border_radius(border_radius, False)

    def get_layout(self):
        return self.self_layout
    
    def set_border_radius(self, radius, update=True):
        self.border_radius = radius
        if update:
            self.update()
        
    def set_border_width(self, width, update=True):
        self.border_width = width
        self.self_layout.setContentsMargins( self.border_width, self.border_width, self.border_width, self.border_width )
        if update:
            self.update()

    def set_background_color(self, color, update=True):
        self.background_color = color
        
        ## without this line it wont paint the background, but the children get the background color info
        ## with this line, the rounded corners will be ruined
        #self.setAttribute(QtCore.Qt.WA_StyledBackground, True)
        self.setStyleSheet('background-color: ' + self.background_color.name())
        
        if update:            
            self.update()
    
    def paintEvent(self, event):
        s = self.size()
        qp = QPainter()
        qp.begin(self)
        qp.setRenderHint(QPainter.Antialiasing, True)

        #if self.background_color:        
        qp.setBrush( self.background_color )
        qp.drawRoundedRect(0, 0, s.width(), s.height(), self.border_radius, self.border_radius)
        qp.end()    
    


# ==================
#
# Card
#
# ==================
class Card(QWidget):

    STATUS_NORMAL = 0
    STATUS_SELECTED = 1
    STATUS_DISABLED = 2
    
    DEFAULT_BORDER_WIDTH = 2
    DEFAULT_BORDER_RADIUS = 10
    
    DEFAULT_BORDER_NORMAL_COLOR = None
    DEFAULT_BORDER_SELECTED_COLOR = None
    DEFAULT_BORDER_DISABLED_COLOR = None
    
    DEFAULT_STATUS = STATUS_NORMAL
    
    onMouseClicked = pyqtSignal(int)
    #onMouseDragged = pyqtSignal(int)
    
    def __init__(self, card_holder, card_data, local_index, index):
        QWidget.__init__(self, card_holder)

        self.card_data = card_data
        self.index = index
        self.local_index = local_index
        self.card_holder = card_holder
#        self.actual_position = 0
        
        self.self_layout = QVBoxLayout(self)
        self.setLayout(self.self_layout)
        #self.self_layout.setContentsMargins(self.border_width,self.border_width,self.border_width,self.border_width)
        self.self_layout.setSpacing(0)
        
        self.set_border_normal_color(Card.DEFAULT_BORDER_NORMAL_COLOR)
        self.set_border_selected_color(Card.DEFAULT_BORDER_SELECTED_COLOR)
        self.set_border_disabled_color(Card.DEFAULT_BORDER_DISABLED_COLOR)
        
        ## without this line it wont paint the background, but the children get the background color info
        ## with this line, the rounded corners will be ruined
        #self.setAttribute(QtCore.Qt.WA_StyledBackground, True)
        #self.setStyleSheet('background-color: ' + "yellow")  

        # Panel where the content could be placed
        self.panel = Panel(self)
        self.panel_layout = self.panel.get_layout()
        self.self_layout.addWidget(self.panel)
        
        self.border_radius = Card.DEFAULT_BORDER_RADIUS
        self.set_border_width(Card.DEFAULT_BORDER_WIDTH, False)
        self.set_border_radius(Card.DEFAULT_BORDER_RADIUS, False)        
        #self.set_rate_of_width_decline(Card.DEFAULT_RATE_OF_WIDTH_DECLINE, False)
        
        self.set_status(Card.STATUS_NORMAL)
        
        # Connect the widget to the Container's Resize-Event
        self.card_holder.resized.connect(self.resized)
        
        self.drag_start_position = None
        self.already_mouse_pressed = False
        #self.setDragEnabled(True)
        #self.setAcceptDrops(True)
      
      
        self.onMouseClicked.connect(self.card_holder.animated_move_to)
        #self.onMouseDragged.connect(self.card_holder.rolling_wheel)
 
    def get_card_holder(self):
        return self.card_holder
    
    def set_selected(self):
        self.set_status(Card.STATUS_SELECTED, True)
        
    def set_not_selected(self):
        self.set_status(Card.STATUS_NORMAL, True)
        
    def set_status(self, status, update=False):
        self.status = status

        if status == Card.STATUS_NORMAL:        
            self.set_border_color(self.get_border_normal_color(), update)
        elif status == Card.STATUS_SELECTED:
            self.set_border_color(self.get_border_selected_color(), update)
        elif status == Card.STATUS_DISABLED:
            self.set_border_color(self.get_border_disabled_color(), update)

    def get_status(self):
        return self.status

    def is_selected(self):
        return True if self.status == Card.STATUS_SELECTED else False

    def refresh_color(self):
        self.update()

    def set_border_color(self, color, update):
        self.border_color = color
        if update:
            self.update()

    def get_border_normal_color(self):
        return self.border_normal_color
    
    def get_border_selected_color(self):
        return self.border_selected_color
    
    def get_border_disabled_color(self):
        return self.border_disabled_color
    
    def set_border_normal_color(self, color):
        self.border_normal_color = color
        
    def set_border_selected_color(self, color):
        self.border_selected_color = color
        
    def set_border_disabled_color(self, color):
        self.border_disabled_color = color
 
    def set_background_color(self, color):
        self.panel.set_background_color(color)

    def set_border_width(self, width, update=True):
        self.border_width = width
        self.self_layout.setContentsMargins(self.border_width,self.border_width,self.border_width,self.border_width)
        self.panel.set_border_radius(max(0, self.border_radius - self.border_width), update)
        if update:
            self.update()

    def set_border_radius(self, radius, update=True):
        self.border_radius = radius
        self.panel.set_border_radius(max(0, self.border_radius - self.border_width), update)
        if update:
            self.update()

    def get_panel(self):
        return self.panel
 
    def paintEvent(self, event):
        s = self.size()
        qp = QPainter()
        qp.begin(self)
        qp.setRenderHint(QPainter.Antialiasing, True)
        
        if self.border_color:
            qp.setBrush(self.border_color)
            #qp.setPen( QPen(Qt.black, 0) )
            qp.drawRoundedRect(0, 0, s.width(), s.height(), self.border_radius, self.border_radius)
        
        qp.end()
 
 
    # --------------
    #
    # MOUSE handling
    #
    # --------------
    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            
            # indicates that the mouse button was pressed
            self.already_mouse_pressed = True
            
            self.drag_start_position = event.pos()
            
        event.ignore()

    def mouseMoveEvent(self, event):
        if event.buttons() == Qt.LeftButton:
            self.already_mouse_pressed = False
        event.ignore()
       

    def mouseReleaseEvent(self, event):
        if event.button() == Qt.LeftButton:
            if self.already_mouse_pressed and self.local_index > 0:                
                self.onMouseClicked.emit(self.local_index)
        event.ignore()








 
 
 
    # ----------------------------------------
    #
    # Resize the width of the Card
    #
    # It is called when:
    # 1. CardHolder emits a resize event
    # 2. The Card is created and Placed
    #
    # ----------------------------------------
    def resized(self, width, height, local_index=None):
        # The farther the card is the narrower it is
        if local_index==None:
            local_index = self.local_index
        standard_width = width - 2*self.card_holder.get_border_width() - 2*self.card_holder.get_x_offset_by_index_method(local_index)
        self.resize( standard_width, self.sizeHint().height() )

    # ---------------------------------------
    #
    # Place the Card into the given position
    #
    # 0. position means the card is in front
    # 1. position is behind the 0. position
    # and so on
    # ---------------------------------------
    def place(self, local_index, front_remove=False):
        
        # Need to resize and reposition the Car as 'The farther the card is the narrower it is'
        self.resized(self.card_holder.width(), self.card_holder.height(), local_index)
        x_coordinate = self.card_holder.get_border_width() + self.card_holder.get_x_offset_by_index_method(local_index)
        y_coordinate = self.card_holder.get_border_width() + self.get_y_coordinate(local_index)
        
        if front_remove:
            y_coordinate = y_coordinate - local_index * (math.exp(5 - 5 * local_index) / 2000) * self.height()
            
        self.move( x_coordinate, y_coordinate )
        
        self.show()
        
        return(QPoint(x_coordinate, y_coordinate), QPoint(self.width(),self.height()))

    # ----------------------------------
    #
    # Get Y coordinate by the local_index
    #
    # ----------------------------------
    def get_y_coordinate(self, local_index):
        max_card = min(self.card_holder.max_overlapped_cards, len(self.card_holder.card_descriptor_list) - 1)
        
        dif = self.card_holder.max_overlapped_cards - max_card
        
        reverse_index = max_card - min(local_index, max_card)  #0->most farther
        return self.card_holder.get_y_coordinate_by_reverse_index_method(reverse_index, dif)

    
    













# =========================
#
# Collect Cards
#
# ========================= 
class CollectCardsThread(QtCore.QThread):
    
    collectionStarted = pyqtSignal()
    collectionFinished = pyqtSignal(list)    
    
    __instance = None
    __run = False

    def __new__(cls):
        if cls.__instance is None:
            cls.__instance = super().__new__(cls)
        return cls.__instance    

    @classmethod
    def get_instance(cls, parent, collect_cards_method, paths=None):
        if not cls.__run:
            inst = cls.__new__(cls)
            cls.__init__(cls.__instance, parent, collect_cards_method, paths) 
            return inst
        else:
            return None

    def __init__(self, parent, collect_cards_method, paths):
        QThread.__init__(self)
        
        self.parent = parent
        self.collect_cards_method = collect_cards_method
        self.paths = paths
        
    def run(self):
        CollectCardsThread.__run = True

        # Emits the collectionStarted Event
        self.collectionStarted.emit()

        #### for TEST reason
        #time.sleep(5)
        ####
        
        # Here Collects the Cards Info
        card_list = self.collect_cards_method(self.paths)
        
        # Emits the collectionFinished Event
        self.collectionFinished.emit(card_list)

        # Indicates that the Thread is ready to start again
        CollectCardsThread.__run = False

    def __del__(self):
        self.exiting = True
        self.wait()
 
# =========================
#
# Rolling Animation
#
# ========================= 
class AnimateRolling(QThread):
    
    positionChanged = pyqtSignal(int)
    __instance = None
    __run = False
    
    def __new__(cls):
        if cls.__instance is None:
            cls.__instance = super().__new__(cls)
        return cls.__instance    
    
    @classmethod
    def get_instance(cls, loop, value, sleep=0.01):
        if not cls.__run:
            inst = cls.__new__(cls)
            cls.__init__(cls.__instance, loop, value, sleep) 
            return inst
        else:
            return None
    
    def __init__(self, loop, value, sleep):
        QThread.__init__(self)
        self.loop = loop
        self.value = value
        self.sleep = sleep
            
    def __del__(self):
        self.wait()
    
    def run(self): 
        
        # blocks to call again
        AnimateRolling.__run = True
        for i in range(self.loop):
            time.sleep(self.sleep)
            self.positionChanged.emit(self.value)
        
        # release blocking
        AnimateRolling.__run = False


# =========================
#
# CountDown Timer
#
# =========================
class CountDown(QThread):
    
    timeOver = pyqtSignal()
    __timer = 0    
    
    def __init__(self):
        QThread.__init__(self)
            
    def __del__(self):
        self.wait()
    
    def run(self): 

        # Ha meg mukodik
        if CountDown.__timer > 0:
            CountDown.__timer = 10

        # Ha mar nem mukodik
        else:
            CountDown.__timer = 10
        
            while CountDown.__timer > 0:
                time.sleep(0.04)
                CountDown.__timer = CountDown.__timer - 1
                #print(CountDown.__timer)
               
            #print("most emital")
            self.timeOver.emit()



import sys
import os
import importlib
from threading import Thread
from pkg_resources import resource_string, resource_filename
from functools import cmp_to_key
import locale

from cardholder.cardholder import CardHolder
from cardholder.cardholder import Card
from cardholder.cardholder import CollectCardsThread

from akoteka.gui.pyqt_import import *
from akoteka.gui.card_panel import CardPanel
from akoteka.gui.configuration_dialog import ConfigurationDialog

from akoteka.accessories import collect_cards
from akoteka.accessories import filter_key
from akoteka.accessories import clearLayout
from akoteka.accessories import FlowLayout


from akoteka.constants import *
from akoteka.setup.setup import getSetupIni

from akoteka.handle_property import _
from akoteka.handle_property import re_read_config_ini
from akoteka.handle_property import config_ini
from akoteka.handle_property import get_config_ini

class GuiAkoTeka(QWidget, QObject):
    
    def __init__(self):
        #super().__init__()  
        QWidget.__init__(self)
        QObject.__init__(self)
        
        self.actual_card_holder = None
        self.card_holder_history = []
        
        # most outer container, just right in the Main Window
        box_layout = QVBoxLayout(self)
        self.setLayout(box_layout)
        # controls the distance between the MainWindow and the added container: scrollContent
        box_layout.setContentsMargins(0, 0, 0, 0)
        box_layout.setSpacing(0)
    
        # control panel
        self.control_panel = ControlPanel(self)
        self.control_panel.set_back_button_method(self.restore_previous_holder)
        box_layout.addWidget( self.control_panel)
    
        # scroll_content where you can add your widgets - has scroll
        scroll = QScrollArea(self)
        box_layout.addWidget(scroll)
        scroll.setWidgetResizable(True)
        scroll_content = QWidget(scroll)
        scroll_content.setStyleSheet('background: ' + COLOR_MAIN_BACKGROUND)
        scroll.setFocusPolicy(Qt.NoFocus)
    
#        scroll_content = QWidget(self)
#        scroll_content.setStyleSheet('background: ' + COLOR_MAIN_BACKGROUND)
#        box_layout.addWidget(scroll_content)
#        scroll_layout = QVBoxLayout(scroll_content)
#        scroll_content.setLayout(scroll_layout)

        # layout of the content with margins
        scroll_layout = QVBoxLayout(scroll_content)        
        scroll.setWidget(scroll_content)        
        # vertical distance between cards - Vertical
        scroll_layout.setSpacing(5)
        # spaces between the added Widget and this top, right, bottom, left side
        scroll_layout.setContentsMargins(15,15,15,15)
        scroll_content.setLayout(scroll_layout)

        # -------------------------------
        # Title
        # -------------------------------
        self.hierarchy_title = HierarchyTitle(scroll_content, self)
        self.hierarchy_title.set_background_color(QColor(COLOR_CARDHOLDER_BACKGROUND))
        self.hierarchy_title.set_border_radius(RADIUS_CARDHOLDER)

        # -------------------------------
        # Here comes later the CardHolder
        # -------------------------------
        self.card_holder_panel = QWidget(scroll_content)
        
        scroll_layout.addWidget(self.hierarchy_title)
        scroll_layout.addWidget(self.card_holder_panel)
        scroll_layout.addStretch(1)
        
        self.card_holder_panel_layout = QVBoxLayout(self.card_holder_panel)
        self.card_holder_panel_layout.setContentsMargins(0,0,0,0)
        self.card_holder_panel_layout.setSpacing(0)

        self.back_button_listener = None

        # --- Window ---
        sp=getSetupIni()
        self.setWindowTitle(sp['name'] + '-' + sp['version'])    
        #self.setGeometry(300, 300, 300, 200)
        self.resize(900,600)
        self.center()
        self.show()    

    def center(self):
        """Aligns the window to middle on the screen"""
        fg=self.frameGeometry()
        cp=QDesktopWidget().availableGeometry().center()
        fg.moveCenter(cp)
        self.move(fg.topLeft())

    # --------------------------
    #
    # Start CardHolder
    #
    # --------------------------
    def startCardHolder(self):

        # Create the first Card Holder - 
        #self.go_down_in_hierarchy( [], "" )
        self.go_down_in_hierarchy() 

        # Retreive the media path
        paths = [config_ini['media_path']]
        
        # Start to collect the Cards from the media path
        self.actual_card_holder.startCardCollection(paths)        

    # --------------------------------------------------------------
    #
    # Go deeper in the hierarchy
    #
    # card_descriptor_structure The NOT Filtered card hierarchy list
    #                           on the recent level
    # title                     The title what whould be shown above
    #                           the CardHolder
    # save                      It controls to save the CardHolder
    #                           into the history list
    #                           collecting_finish uses it with False
    # --------------------------------------------------------------
    def go_down_in_hierarchy( self, card_descriptor_structure=None, title=None, save=True ):

        # if it is called very first time
        if card_descriptor_structure is None:
            self.initialize = True
            card_descriptor_structure = []
            title = ""
            save = False
        else:
            self.initialize = False

        # if there is already a CardHolder
        if self.actual_card_holder:

            # hide the old CardHolder
            self.actual_card_holder.setHidden(True)

        # if it is said to Save the CardHolder to the history list
        if save:
            
            # save the old CardHolder it in a list
            self.card_holder_history.append(self.actual_card_holder)
                    
        self.actual_card_holder = CardHolder(            
            self, 
            self.get_new_card,
            self.collect_cards,
            self.collecting_start,
            self.collecting_finish
        )
        
        self.actual_card_holder.title = title
        self.actual_card_holder.set_max_overlapped_cards( MAX_OVERLAPPED_CARDS )
        self.actual_card_holder.set_y_coordinate_by_reverse_index_method(self.get_y_coordinate_by_reverse_index)
        self.actual_card_holder.set_x_offset_by_index_method(self.get_x_offset_by_index)
        self.actual_card_holder.set_background_color(QColor(COLOR_CARDHOLDER_BACKGROUND))
        self.actual_card_holder.set_border_radius(RADIUS_CARDHOLDER)
        self.actual_card_holder.set_border_width(15)        
                
        # Save the original card desctiptor structure into the CardHolder
        self.actual_card_holder.orig_card_descriptor_structure = card_descriptor_structure
        
        # Make the CardHolder to be in Focus
        self.actual_card_holder.setFocus()

        # Set the title of the CardHolder - The actual level of the hierarchy
        self.hierarchy_title.set_title(self.card_holder_history, self.actual_card_holder)

        # add the new holder to the panel
        self.card_holder_panel_layout.addWidget(self.actual_card_holder)
        #self.scroll_layout.addStretch(1)
        
        # filter the list by the filters + Fill-up the CardHolder with Cards using the parameter as list of descriptor
        self.filter_the_cards(card_descriptor_structure)

    # -------------------------
    #
    # Come up in the hierarchy
    #
    # -------------------------
    def restore_previous_holder(self, steps=1):
        
        size = len(self.card_holder_history)
        if  size >= steps:

            for i in range(0, steps):
            
                previous_card_holder = self.card_holder_history.pop()
                # get the previous CardHolder
                #previous_card_holder = self.card_holder_history[size - 1]
            
                # remove the previous CardHolder from the history list
                #self.card_holder_history.remove(previous_card_holder)
            
            # hide the old CardHolder
            self.actual_card_holder.setHidden(True)            
            
            # remove from the layout the old CardHolder
            self.card_holder_panel_layout.removeWidget(self.actual_card_holder)
        
            # the current card holder is the previous
            self.actual_card_holder = previous_card_holder
            
            # show the current card holder
            self.actual_card_holder.setHidden(False)

            # set the title
            self.hierarchy_title.set_title(self.card_holder_history, self.actual_card_holder)
            
            # filter the list by the filters + Fill-up the CardHolder with Cards using the parameter as list of descriptor
            self.filter_the_cards(self.actual_card_holder.orig_card_descriptor_structure)
            
            # select the Card which was selected to enter
            self.actual_card_holder.select_actual_card()

    # ------------------
    # Collecting Started
    # ------------------
    def collecting_start(self):
        """
        Indicates that the CardCollection process started.
        The CardHolder calls this method
        Jobs to do:
            -Hide the title
        """
        self.hierarchy_title.setHidden(True)

    # -------------------
    # Collecting Finished
    # -------------------
    def collecting_finish(self, card_holder, card_descriptor_structure):
        """
        Indicates that the CardCollection process finished.
        The CardHolder calls this method
        Jobs to do:
            -Show the title
            -Set up the filters
        """        
        
        if card_descriptor_structure:
          
            # Show the title of the CardHolder (the certain level)        
            self.hierarchy_title.setHidden(False)
       
        
        # Save the NOT Filtered list
        card_holder.orig_card_descriptor_structure = card_descriptor_structure
        
        # Set-up the Filters        
        self.set_up_filters(card_descriptor_structure)
        
        # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        # This part is tricky
        # It prevents to show the 0. level of Cards
        # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        #
        # if there is only ONE Card and the status of the presentation is "initialize"
        if len(card_holder.card_descriptor_list) == 1 and self.initialize:
            # then I go down ONE level
            self.go_down_in_hierarchy(card_holder.card_descriptor_list[0]['extra']["orig-sub-cards"], card_holder.card_descriptor_list[0]['title'][config_ini['language']], save=False )

    # ----------------------------------------------------------
    #
    # Calculates the Y coordinate of a Card using reverse index
    #
    # ----------------------------------------------------------
    def get_y_coordinate_by_reverse_index(self, reverse_index, diff_to_max):
        raw_pos = (reverse_index + diff_to_max) * (reverse_index + diff_to_max)
        offset = diff_to_max * diff_to_max
        return (raw_pos - offset) * 8
        #return reverse_index * 220
    
    # -----------------------------------------------
    #
    # Calculates the X coordinate of a Card by index
    #
    # -----------------------------------------------
    def get_x_offset_by_index(self, index):
        return index * 4       

    def collect_cards(self, paths):
        cdl = collect_cards(paths)
        
        # Preparation for collecting the filtered_card_structure and filters
        filtered_card_structure = []
        filter_hit_list = {
            "genre": set(),
            "theme": set(),
            "director": set(),
            "actor": set(),
            "favorite": set(),
            "new": set(),
            "best": set()
        }
        self.generate_filtered_card_structure(cdl, filtered_card_structure, filter_hit_list)
        
        return filtered_card_structure
    
    # --------------------
    #
    # Generates a new Card
    #
    # --------------------
    def get_new_card(self, card_data, local_index, index):

        card = Card(self.actual_card_holder, card_data, local_index, index)
        
        if card_data["extra"]["media-path"]:
            card.set_background_color(QColor(COLOR_CARD_MOVIE_BACKGROUND))
        else:
            card.set_background_color(QColor(COLOR_CARD_COLLECTOR_BACKGROUND))
            
        card.set_border_normal_color(QColor(COLOR_CARD_BORDER_NORMAL_BACKGROUND))
        card.set_border_selected_color(QColor(COLOR_CARD_BORDER_SELECTED_BACKGROUND))
        
        card.set_not_selected()
        card.set_border_radius( RADIUS_CARD )
        card.set_border_width( WIDTH_BORDER_CARD )
 
        panel = card.get_panel()        
        layout = panel.get_layout()
        layout.setContentsMargins(0, 0, 0, 0)

        card_panel = CardPanel(card, card_data)
        layout.addWidget(card_panel)
        
        return card
  
    def set_filter_listener(self, listener):
        self.control_panel.set_filter_listener(listener)
        
    def get_filter_holder(self):
        return self.control_panel.get_filter_holder()
      
    # --------------------------------
    #
    # Filter the Cards
    #
    # Filters the Cards and Show them
    #
    # --------------------------------
    def filter_the_cards(self, card_descriptor_structure=None):
        if card_descriptor_structure is None:
            card_descriptor_structure = self.actual_card_holder.orig_card_descriptor_structure
        filtered_card_structure = self.set_up_filters(card_descriptor_structure)
        self.actual_card_holder.fillUpCardHolderByDescriptor(filtered_card_structure)

    # ----------------
    # Set-up Filters
    # ----------------
    def set_up_filters(self, card_descriptor_structure):
        """
        Based on the list that received as parameter, 
        it selects the possible filter elements
        """
        
        # ###################################
        # Turn OFF the listener to the Filter
        # ###################################
        self.set_filter_listener(None)
        
        # ####################################
        # Save the recent state of the filters
        # ####################################
        filters = {
            "genre": "",
            "theme": "",
            "director": "",
            "actor": "",
            "favorite": "",
            "new": "",
            "best": ""
        }
        for category, value in self.get_filter_holder().get_filter_selection().items():            
            if value != None and value != "":
                filters[category] = value
        
        # #############
        # Setup Filters
        # #############

        # Preparation for collecting the filtered_card_structure and filters
        filtered_card_structure = []
        filter_hit_list = {
            "genre": set(),
            "theme": set(),
            "director": set(),
            "actor": set(),
            "favorite": set(),
            "new": set(),
            "best": set()
        }
        self.generate_filtered_card_structure(card_descriptor_structure, filtered_card_structure, filter_hit_list)
        
        # Fill up GENRE dropdown
        self.get_filter_holder().clear_genre()
        self.get_filter_holder().add_genre("", "")

        for element in sorted([(_("genre_" + e),e) for e in filter_hit_list['genre']], key=lambda t: locale.strxfrm(t[0]) ):            
            self.get_filter_holder().add_genre(element[0], element[1])
        
        # Fill up THEME dropdown
        self.get_filter_holder().clear_theme()
        self.get_filter_holder().add_theme("", "")
        for element in sorted([(_("theme_" + e), e) for e in filter_hit_list['theme']], key=lambda t: locale.strxfrm(t[0]) ):            
            self.get_filter_holder().add_theme(element[0], element[1])

        # Fill up DIRECTOR dropdown
        self.get_filter_holder().clear_director()
        self.get_filter_holder().add_director("")
        for element in sorted( filter_hit_list['director'], key=cmp_to_key(locale.strcoll) ):
            self.get_filter_holder().add_director(element)

        # Fill up ACTOR dropdown
        self.get_filter_holder().clear_actor()
        self.get_filter_holder().add_actor("")
        for element in sorted( filter_hit_list['actor'], key=cmp_to_key(locale.strcoll) ):
            self.get_filter_holder().add_actor(element)
        
        # ####################
        # Reselect the Filters
        # ####################
        self.get_filter_holder().select_genre(filters["genre"])
        self.get_filter_holder().select_theme(filters["theme"])
        self.get_filter_holder().select_director(filters["director"])
        self.get_filter_holder().select_actor(filters["actor"])
        
        # #######################################
        # Turn back ON the listener to the Filter
        # #######################################
        self.set_filter_listener(self)

        return filtered_card_structure
    
    
    # ================================
    # 
    # Generates Filtered CardStructure
    #
    # ================================   
    def generate_filtered_card_structure(self, card_structure, filtered_card_structure, filter_hit_list):
        """
        This method serves a dual task:
            -Based on the Filter it generates a new, filtered list: filtered_card_structure
            -Collects the new Filter, based on the filtered list:   filter_hit_list
        """
        mediaFits = False
        collectorFits = False
            
        # through the SORTED list
        for crd in sorted(card_structure, key=lambda arg: arg['title'][config_ini['language']], reverse=False):
            
            card = {}
            card['title'] = crd['title']
            card['storyline'] = crd['storyline']
            card['general'] = crd['general']
            card['rating'] = crd['rating']
            card['links'] = crd['links']

            card['extra'] = {}            
            card['extra']['image-path'] = crd['extra']['image-path']
            card['extra']['media-path'] = crd['extra']['media-path']
            card['extra']['recent-folder'] = crd['extra']['recent-folder']            
            card['extra']['sub-cards'] = [] #json.loads('[]')
            card['extra']['orig-sub-cards'] = crd['extra']['sub-cards']

            # in case of MEDIA CARD
            if crd['extra']['media-path']:

                fits = True
                
                # go through the filters
                for category, value in self.get_filter_holder().get_filter_selection().items():
            
                    # if the specific filter is set
                    if value != None and value != "":

                        if filter_key[category]['store-mode'] == 'v':
                            if value != crd[filter_key[category]['section']][category]:
                                fits = False
                                
                        elif filter_key[category]['store-mode'] == 'a':
                            if value not in crd[filter_key[category]['section']][category]:
                                fits = False
                        else:
                            fits = False

                # let's keep this MEDIA CARD as it fits
                if fits:

                    # Collect the filters
                    for category, value in self.get_filter_holder().get_filter_selection().items():
                        if filter_key[category]['store-mode'] == 'v':
                            if card[filter_key[category]['section']][category]:
                                filter_hit_list[category].add(card[filter_key[category]['section']][category])
                                
                        elif filter_key[category]['store-mode'] == 'a':
                            for cat in card[filter_key[category]['section']][category]:
                                if cat.strip():
                                    filter_hit_list[category].add(cat.strip())
                    
                    filtered_card_structure.append(card)                    
                    mediaFits = True
                    
            # in case of COLLECTOR CARD
            else:                     
                     
                # then it depends on the next level
                fits = self.generate_filtered_card_structure(crd['extra']['sub-cards'], card['extra']['sub-cards'], filter_hit_list)
                
                if fits:
                    filtered_card_structure.append(card)
                    collectorFits = True
        
        return (mediaFits or collectorFits)
  
    # ----------------------------
    #
    # Key Press Event: Enter/Space
    #
    # ----------------------------
    def keyPressEvent(self, event):

        #
        # Enter / Return / Space
        #
        if event.key() == Qt.Key_Return or event.key() == Qt.Key_Enter or event.key() == Qt.Key_Space:

            # Simulate a Mouse Press / Release Event on the Image
            if self.actual_card_holder.shown_card_list and len(self.actual_card_holder.shown_card_list) > 0:
                card=self.actual_card_holder.shown_card_list[0]
                if card.is_selected():
                    
                    event_press = QMouseEvent(QEvent.MouseButtonPress, QPoint(10,10), Qt.LeftButton, Qt.LeftButton, Qt.NoModifier)
                    event_release = QMouseEvent(QEvent.MouseButtonRelease, QPoint(10,10), Qt.LeftButton, Qt.LeftButton, Qt.NoModifier)

                    layout=card.get_panel().get_layout()                    
                    card_panel = layout.itemAt(0).widget()
                    card_panel.card_image.mousePressEvent(event_press)
                    card_panel.card_image.mouseReleaseEvent(event_release)            

        #
        # Escape
        #
        if event.key() == Qt.Key_Escape:
            self.restore_previous_holder()

        event.ignore()
  

class LinkLabel(QLabel):
    def __init__(self, text, parent, index):
        QLabel.__init__(self, text, parent)
        self.parent = parent
        self.index = index        
        self.setFont(QFont( FONT_TYPE, HIERARCHY_TITLE_FONT_SIZE, weight=QFont.Bold if index else QFont.Normal))

    # Mouse Hover in
    def enterEvent(self, event):
        if self.index:
            QApplication.setOverrideCursor(Qt.PointingHandCursor)
        event.ignore()

    # Mouse Hover out
    def leaveEvent(self, event):
        if self.index:
            QApplication.restoreOverrideCursor()
        event.ignore()

    # Mouse Press
    def mousePressEvent(self, event):
        if event.buttons() == Qt.LeftButton and self.index:
            self.parent.panel.restore_previous_holder(self.index)
        event.ignore()
    

# =========================================
# 
# This Class represents the title
#
# =========================================
#
class HierarchyTitle(QWidget):
    DEFAULT_BACKGROUND_COLOR = Qt.lightGray
    DEFAULT_BORDER_RADIUS = 10
    
    def __init__(self, parent, panel):
        QWidget.__init__(self, parent)

        self.panel = panel
        
        self_layout = QHBoxLayout(self)
        self_layout.setContentsMargins(5, 5, 5, 5)
        self_layout.setSpacing(0)
        #self_layout.setAlignment(Qt.AlignHCenter)
        self.setLayout(self_layout)

        self.text = QWidget(self)
        #self.text.setWordWrap(True)
        #self.text.setAlignment(Qt.AlignHCenter)
        #self.text.setFont(QFont( FONT_TYPE, HIERARCHY_TITLE_FONT_SIZE, weight=QFont.Bold))
        self_layout.addWidget(self.text)
        
        #self.text_layout = FlowLayout(self.text)
        self.text_layout = QVBoxLayout(self.text)
        self.text_layout.setContentsMargins(0, 0, 0, 0)
        self.text_layout.setSpacing(0)
        self.text.setLayout(self.text_layout)

        self.set_background_color(QColor(HierarchyTitle.DEFAULT_BACKGROUND_COLOR), False)
        self.set_border_radius(HierarchyTitle.DEFAULT_BORDER_RADIUS, False)
        
        self.card_holder_history = None
        self.actual_card_holder = None
        self.no_refresh = False
        
#    def resizeEvent(self, event):
#        if not self.no_refresh:
#            self.refresh_title()
#        event.accept()
    
    def refresh_title(self):
        if self.card_holder_history and self.actual_card_holder:
            self.set_title(self.card_holder_history, self.actual_card_holder)
    
    def set_title(self, card_holder_history, actual_card_holder):
        #self.blockSignals(True)
        #self.no_refresh = True

        clearLayout(self.text_layout)
 
        self.card_holder_history = card_holder_history
        self.actual_card_holder = actual_card_holder
 
        history = []
        for index, card in enumerate(card_holder_history):
            if card.title:
                label = LinkLabel(card.title, self, len(card_holder_history)-index)
                history.append(label)
       
        minimumWidth = 0
        max_width = self.size().width() - 2 * 5        
        self.create_one_line_container()
        
        for cw in history:
            
            minimumWidth += self.get_width_in_pixels(cw)
            if minimumWidth <= max_width:            
                self.add_to_one_line_container(cw)
            else:
                self.push_new_line_container()
                self.create_one_line_container()
                minimumWidth = 0
                self.add_to_one_line_container(cw)
            
            separator = QLabel(' > ')
            separator.setFont(QFont( FONT_TYPE, HIERARCHY_TITLE_FONT_SIZE, weight=QFont.Normal ))
            minimumWidth += self.get_width_in_pixels(separator)
            if minimumWidth <= max_width:            
                self.add_to_one_line_container(separator)
            else:
                self.push_new_line_container()
                self.create_one_line_container()
                minimumWidth = 0
                self.add_to_one_line_container(separator)
                                       
        notLinkTitle = LinkLabel(actual_card_holder.title, self, 0)
        minimumWidth += self.get_width_in_pixels(notLinkTitle)
        if minimumWidth <= max_width:            
            self.add_to_one_line_container(notLinkTitle)
        else:
            self.push_new_line_container()
            self.create_one_line_container()
            minimumWidth = 0
            self.add_to_one_line_container(notLinkTitle)
        self.push_new_line_container()
       
        #self.no_refresh = False
        
        #self.blockSignals(False)
       
        #print(self.panel.size().width(), minimumWidth)
        #self.text_layout.addWidget(LinkLabel(actual_card_holder.title, self, 0))
        #print(self.panel.size().width())
        #print()



    def create_one_line_container(self):
        self.one_line_container = QWidget(self)
        self.one_line_container_layout = QHBoxLayout(self.one_line_container)
        self.one_line_container_layout.setContentsMargins(0, 0, 0, 0)
        self.one_line_container_layout.setSpacing(0)
        self.one_line_container.setLayout(self.one_line_container_layout)
        self.one_line_container_layout.setAlignment(Qt.AlignHCenter)

    def add_to_one_line_container(self, cw):
        self.one_line_container_layout.addWidget(cw)
        
    def push_new_line_container(self):
        self.text_layout.addWidget(self.one_line_container)
        
    def get_width_in_pixels(self, cw):
        initialRect = cw.fontMetrics().boundingRect(cw.text());
        improvedRect = cw.fontMetrics().boundingRect(initialRect, 0, cw.text());   
        return improvedRect.width()
        
        
    def set_background_color(self, color, update=False):
        self.background_color = color
        self.text.setStyleSheet('background: ' + color.name()) 
        if update:
            self.update()
            
    def set_border_radius(self, radius, update=True):
        self.border_radius = radius
        if update:
            self.update()            
        
    def paintEvent(self, event):
        s = self.size()
        qp = QPainter()
        qp.begin(self)
        qp.setRenderHint(QPainter.Antialiasing, True)
        qp.setBrush( self.background_color )

        qp.drawRoundedRect(0, 0, s.width(), s.height(), self.border_radius, self.border_radius)
        qp.end()
        
        

# =========================================
#
#          Control Panel 
#
# on the TOP of the Window
#
# Contains:
#           Back Button
#           Filter
#
# =========================================
class ControlPanel(QWidget):
    def __init__(self, gui):
        super().__init__()
       
        self.gui = gui
        
        self_layout = QHBoxLayout(self)
        self.setLayout(self_layout)
        
        # controls the distance between the MainWindow and the added container: scrollContent
        self_layout.setContentsMargins(3, 3, 3, 3)
        self_layout.setSpacing(5)

        # -------------
        #
        # Back Button
        #
        # -------------     
        self.back_button_method = None
        back_button = QPushButton()
        back_button.setFocusPolicy(Qt.NoFocus)
        back_button.clicked.connect(self.back_button_on_click)
        
        back_button.setIcon( QIcon( resource_filename(__name__,os.path.join("img", IMG_BACK_BUTTON)) ))
        back_button.setIconSize(QSize(32,32))
        back_button.setCursor(QCursor(Qt.PointingHandCursor))
        back_button.setStyleSheet("background:transparent; border:none") 

        # Back button on the left
        self_layout.addWidget( back_button )

        self_layout.addStretch(1)
        
        # -------------------
        #
        # Config Button
        #
        # -------------------
        self.config_button = QPushButton()
        self.config_button.setFocusPolicy(Qt.NoFocus)
        self.config_button.setCheckable(False)
        self.config_button.clicked.connect(self.config_button_on_click)
        
        config_icon = QIcon()
        config_icon.addPixmap(QPixmap( resource_filename(__name__,os.path.join("img", IMG_CONFIG_BUTTON)) ), QIcon.Normal, QIcon.On)
        self.config_button.setIcon( config_icon )
        self.config_button.setIconSize(QSize(25,25))
        self.config_button.setCursor(QCursor(Qt.PointingHandCursor))
        self.config_button.setStyleSheet("background:transparent; border:none") 
        self_layout.addWidget( self.config_button )
        
        
        
        # -------------
        #
        # Filter Holder
        #
        # -------------
        self.filter_holder = FilterHolder()
        self.filter_holder.changed.connect(self.filter_on_change)
        
        # Filter on the right
        self_layout.addWidget(self.filter_holder)

        # Listeners
        self.back_button_listener = None
        self.filter_listener = None

    def refresh_label(self):
        self.filter_holder.refresh_label()

    def set_back_button_method(self, method):
        self.back_button_method = method
        
    def set_filter_listener(self, listener):
        self.filter_listener = listener
        
    def back_button_on_click(self):
        if self.back_button_method:
            self.back_button_method()


    # -----------------------
    #
    # Config Button Clicked
    #
    # -----------------------
    def config_button_on_click(self):

        dialog = ConfigurationDialog()

        # if OK was clicked
        if dialog.exec_() == QDialog.Accepted:        

            # get the values from the DIALOG
            l = dialog.get_language()
            mp = dialog.get_media_path()
            vp = dialog.get_media_player_video()
            vpp = dialog.get_media_player_video_param()
            ap = dialog.get_media_player_audio()
            app = dialog.get_media_player_audio_param()

            # Update the config.ini file
            config_ini_function = get_config_ini()
            config_ini_function.set_media_path(mp) 
            config_ini_function.set_language(l)
            config_ini_function.set_media_player_video(vp)
            config_ini_function.set_media_player_video_param(vpp)
            config_ini_function.set_media_player_audio(ap)
            config_ini_function.set_media_player_audio_param(app)


#!!!!!!!!!!!!
            # Re-read the config.ini file
            re_read_config_ini()

            # Re-import card_holder_pane
            mod = importlib.import_module("akoteka.gui.card_panel")
            importlib.reload(mod)
#!!!!!!!!!!!!

            # remove history
            for card_holder in self.gui.card_holder_history:
                card_holder.setHidden(True)
                self.gui.card_holder_panel_layout.removeWidget(card_holder)
                #self.gui.scroll_layout.removeWidget(card_holder)
            self.gui.card_holder_history.clear()
                
            # Remove recent CardHolder as well
            self.gui.actual_card_holder.setHidden(True)
            self.gui.card_holder_panel_layout.removeWidget(self.gui.actual_card_holder)
            self.gui.actual_card_holder = None
            
            # reload the cards
            self.gui.startCardHolder()
            
            # refresh the Control Panel
            self.refresh_label()
            
        dialog.deleteLater()
 
    def filter_on_change(self):
        if self.filter_listener:
            self.filter_listener.filter_the_cards()
    
    def get_filter_holder(self):
        return self.filter_holder









# ================
#
# Dropdown HOLDER
#
# ================
class FilterDropDownHolder(QWidget):
    
    def __init__(self):
        super().__init__()

        self.self_layout = QVBoxLayout(self)
        self.setLayout( self.self_layout )
        self.self_layout.setContentsMargins(0, 0, 0, 0)
        self.self_layout.setSpacing(1)

#        self.setStyleSheet( 'background: green')

    def add_dropdown(self, filter_dropdown):
        self.self_layout.addWidget(filter_dropdown)

# =============================
#
# Filter Drop-Down Simple
#
# =============================
#
class FilterDropDownSimple(QWidget):
    
    state_changed = pyqtSignal()
    
    def __init__(self, label):
        super().__init__()

        self_layout = QHBoxLayout(self)
        self_layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout( self_layout )
#        self.setStyleSheet( 'background: green')
        
        self.label_widget = QLabel(label)
        self_layout.addWidget( self.label_widget )

        self.dropdown = QComboBox(self)
        self.dropdown.setFocusPolicy(Qt.NoFocus)
        #self.dropdown.setEditable(True)
        
        self.dropdown.currentIndexChanged.connect(self.current_index_changed)

        # TODO does not work to set the properties of the dropdown list. find out and fix
        style =             '''
            QComboBox { max-width: 200px; min-width: 200px; min-height: 15px; max-height: 15px;}
            QComboBox QAbstractItemView::item { min-width:100px; max-width:100px; min-height: 150px;}
            QListView::item:selected { color: red; background-color: lightgray; min-width: 1000px;}"
            '''            

        style_down_arrow = '''
            QComboBox::down-arrow { 
                image: url( ''' + resource_filename(__name__,os.path.join("img", "back-button.jpg")) + ''');
                
            }
        '''
        style_box = '''
            QComboBox { 
                max-width: 200px; min-width: 200px; border: 1px solid gray; border-radius: 5px;
            }
        '''
#       max-width: 200px; min-width: 200px; min-height: 1em; max-height: 1em; border: 1px solid gray; border-radius: 5px;
        
        style_drop_down ='''
            QComboBox QAbstractItemView::item { 
                color: red;
                max-height: 15px;
            }
        '''            
      
        self.dropdown.setStyleSheet(style_box + style_drop_down)
        self.dropdown.addItem("")

        self_layout.addWidget( self.dropdown )
    
    def clear_elements(self):
        self.dropdown.clear()

    def add_element(self, value, id):
        self.dropdown.addItem(value, id)

    # -------------------------------------
    # get the index of the selected element
    # -------------------------------------
    def get_selected_index(self):
        return self.dropdown.itemData( self.dropdown.currentIndex() )

    # -------------------------------------
    # get the value of the selected element
    # -------------------------------------
    def get_selected_value(self):
        return self.dropdown.itemText( self.dropdown.currentIndex() )
    
    def select_element(self, id):
        self.dropdown.setCurrentIndex( self.dropdown.findData(id) )

    def current_index_changed(self):
        self.state_changed.emit()
        
    def refresh_label(self, new_label):
        self.label_widget.setText(new_label)


# ==========
#
# CheckBox
#
# ==========
class FilterCheckBox(QCheckBox):
    def __init__(self, label):
        super().__init__(label)

        self.setLayoutDirection( Qt.RightToLeft )
        style_checkbox = '''
            QCheckBox { 
                min-height: 15px; max-height: 15px; border: 0px solid gray;
            }
        '''
        self.setStyleSheet( style_checkbox )
#        self.setFocusPolicy(Qt.NoFocus)

    def is_checked(self):
        return 'y' if self.isChecked() else None        
 
    def refresh_label(self, new_label):
        self.setText(new_label)


# ================
#
# Checkbox HOLDER
#
# ================
class FilterCheckBoxHolder(QWidget):
    
    def __init__(self):
        super().__init__()

        self.self_layout = QVBoxLayout(self)
        self.setLayout( self.self_layout )
        self.self_layout.setContentsMargins(0, 0, 0, 0)
        self.self_layout.setSpacing(1)

        #self.setStyleSheet( 'background: green')
        
    def add_checkbox(self, filter_checkbox):
        self.self_layout.addWidget(filter_checkbox)
        

# ===============
#
# Filter HOLDER
#
# ===============
class FilterHolder(QWidget):
    
    changed = pyqtSignal()
    
    def __init__(self):
        super().__init__()

        self_layout = QHBoxLayout(self)
        self.setLayout( self_layout )
        self_layout.setContentsMargins(0, 0, 0, 0)
        self_layout.setSpacing(8)    
        
        # ----------
        #
        # Checkboxes
        #
        # ----------
        self.filter_cb_favorite = FilterCheckBox(_('title_favorite'))
        self.filter_cb_best = FilterCheckBox(_('title_best'))
        self.filter_cb_new = FilterCheckBox(_('title_new'))
                
        holder_checkbox = FilterCheckBoxHolder()
        
        holder_checkbox.add_checkbox(self.filter_cb_favorite)
        holder_checkbox.add_checkbox(self.filter_cb_best)
        holder_checkbox.add_checkbox(self.filter_cb_new)        
                
        # Listener
        self.filter_cb_favorite.stateChanged.connect(self.state_changed)
        self.filter_cb_best.stateChanged.connect(self.state_changed)
        self.filter_cb_new.stateChanged.connect(self.state_changed)
                        
        self_layout.addWidget(holder_checkbox)

        # ----------
        #
        # Dropdowns 
        #
        # ----------

        #
        # Dropdown - genre+theme
        #
        self.filter_dd_genre = FilterDropDownSimple(_('title_genre'))
        self.filter_dd_theme = FilterDropDownSimple(_('title_theme'))
        
        holder_dropdown_gt = FilterDropDownHolder()
        
        holder_dropdown_gt.add_dropdown(self.filter_dd_genre)
        holder_dropdown_gt.add_dropdown(self.filter_dd_theme)
        
        self_layout.addWidget(holder_dropdown_gt)

        #
        # Dropdown - director+actor
        #
        self.filter_dd_director = FilterDropDownSimple(_('title_director'))
        self.filter_dd_actor = FilterDropDownSimple(_('title_actor'))
        
        holder_dropdown_da = FilterDropDownHolder()
        
        holder_dropdown_da.add_dropdown(self.filter_dd_director)
        holder_dropdown_da.add_dropdown(self.filter_dd_actor)
        
        self_layout.addWidget(holder_dropdown_da)


        #mydd = QComboBox(self)
        #self_layout.addWidget(mydd)
        #mydd.setEditable(True)



        # Listeners
        self.filter_dd_genre.state_changed.connect(self.state_changed)
        self.filter_dd_theme.state_changed.connect(self.state_changed)
        self.filter_dd_director.state_changed.connect(self.state_changed)
        self.filter_dd_actor.state_changed.connect(self.state_changed)

    def refresh_label(self):
        self.filter_dd_genre.refresh_label(_('title_genre'))
        self.filter_dd_theme.refresh_label(_('title_theme'))
        self.filter_dd_director.refresh_label(_('title_director'))
        self.filter_dd_actor.refresh_label(_('title_actor'))
        self.filter_cb_favorite.refresh_label(_('title_favorite'))
        self.filter_cb_new.refresh_label(_('title_new'))
        self.filter_cb_best.refresh_label(_('title_best'))
        
        
    def clear_genre(self):
        self.filter_dd_genre.clear_elements()
        
    def add_genre(self, value, id):
        self.filter_dd_genre.add_element(value, id)
        
    def select_genre(self, id):
        self.filter_dd_genre.select_element(id)

    def clear_theme(self):
        self.filter_dd_theme.clear_elements()

    def add_theme(self, value, id):
        self.filter_dd_theme.add_element(value, id)
        
    def select_theme(self, id):
        self.filter_dd_theme.select_element(id)        

    def clear_director(self):
        self.filter_dd_director.clear_elements()
    
    def add_director(self, director):
        self.filter_dd_director.add_element(director, director)
    
    def select_director(self, id):
        self.filter_dd_director.select_element(id)
        
    def clear_actor(self):
        self.filter_dd_actor.clear_elements()

    def add_actor(self, actor):
        self.filter_dd_actor.add_element(actor, actor)
        
    def select_actor(self, id):
        self.filter_dd_actor.select_element(id)        
    
    def get_filter_selection(self):
        filter_selection = {
            "best": self.filter_cb_best.is_checked(),
            "new": self.filter_cb_new.is_checked(),
            "favorite": self.filter_cb_favorite.is_checked(),
            "genre": self.filter_dd_genre.get_selected_index(),
            "theme": self.filter_dd_theme.get_selected_index(),
            "director": self.filter_dd_director.get_selected_value(),
            "actor": self.filter_dd_actor.get_selected_value()
        }
        return filter_selection
    
    def state_changed(self):
        self.changed.emit()

    def closeEvent(self, event):
        print('close filter holder')














        
def main():   
    
    app = QApplication(sys.argv)
    ex = GuiAkoTeka()
    ex.startCardHolder()
    sys.exit(app.exec_())
    
    



import sys
import os
#import json
from subprocess import call
from threading import Thread
from pkg_resources import resource_string, resource_filename
from functools import cmp_to_key
import locale

#from PyQt5.QtCore import pyqtSignal  

from akoteka.accessories import filter_key
from akoteka.accessories import get_pattern_video
from akoteka.accessories import get_pattern_audio

from akoteka.handle_property import _
from akoteka.handle_property import config_ini
from akoteka.handle_property import Property

from akoteka.gui.pyqt_import import QHLine
from akoteka.gui.pyqt_import import *

from akoteka.constants import *

from cardholder.cardholder import Card


# =========================================
# 
# Card Panel
#
# =========================================
class CardPanel(QWidget):
    """
    This class represents the Content of a Card.
    It is divided into three parts. Left to right:
        -Image
        -Information
        -Rating    
    """
    def __init__(self, card, card_data):
        """
        Constructor of the CardPanel
        card:       Card object, which should be used to present data
        card_data:  The descriptor of the Card
        """
        QWidget.__init__(self, card)
        
        self.card = card
        self.card_data = card_data
        self.card_holder = card.get_card_holder()
        self.card_path = None

        panel_layout = QHBoxLayout(self)
        panel_layout.setContentsMargins(WIDTH_PANEL_MARGIN_LEFT, WIDTH_PANEL_MARGIN_TOP, WIDTH_PANEL_MARGIN_RIGHT, WIDTH_PANEL_MARGIN_BOTTOM)
        panel_layout.setSpacing(10)
        self.setLayout( panel_layout )
        #self.setStyleSheet('background: ' + COLOR_CARD_BACKGROUND)
        #self.setStyleSheet('border:none')

        #
        # Containers in the Card
        #
        self.card_image = CardImage( self )
        panel_layout.addWidget( self.card_image )
        self.card_information = CardInformation()
        panel_layout.addWidget( self.card_information )
        self.card_rating = CardRating()
        panel_layout.addWidget( self.card_rating )
 
        self.set_image_path( card_data["extra"]["image-path"] )
        
        self.set_sub_cards( card_data["extra"]["orig-sub-cards"] )
        #self.set_sub_cards( card_data['extra']['sub-cards'])
        
        self.set_media_path( card_data["extra"]["media-path"] )
        
        self.set_title( card_data['title'][config_ini['language']] )
 
        if card_data["extra"]["media-path"]:

            self.add_element_to_collector_line( _("title_year"), card_data["general"]["year"])
            self.add_element_to_collector_line( _("title_length"), card_data["general"]["length"])
            self.add_element_to_collector_line( _("title_country"), ", ".join( [ _("country_" + a) for a in card_data["general"]["country"] ]) )
            self.add_element_to_collector_line( _("title_sound"), ", ".join( [ _("lang_" + a) for a in card_data["general"]["sound"] ]) if card_data["general"]["sound"] != [''] else "")
            self.add_element_to_collector_line( _("title_sub"), ", ".join( [ _("lang_" + a) for a in card_data["general"]["sub"] ]) if card_data["general"]["sub"] != [''] else "" )
 
            self.add_separator()
            if ''.join(card_data["general"]["director"]):
                self.add_info_line( _("title_director"), ", ".join( [ d for d in card_data["general"]["director"] ] ) )
                    
            if ''.join(card_data["general"]["actor"]):
                self.add_info_line( _("title_actor"), ", ".join( [ a for a in card_data["general"]["actor"] ] ) )

            if ''.join(card_data["general"]["genre"]):
                self.add_info_line( _("title_genre"), ", ".join( [ _("genre_"+g ) if g else "" for g in card_data["general"]["genre"] ] ) )
                    
            if ''.join(card_data["general"]["theme"]):
                self.add_info_line( _("title_theme"), ", ".join( [ _("theme_"+a ) if a else "" for a in card_data["general"]["theme"] ] ) )
                
            if card_data['storyline'][config_ini['language']]:
                self.add_separator()                
                self.add_info_line( _('title_storyline'), card_data['storyline'][config_ini['language']])
                   
            self.add_info_line_stretch()

            self.set_rating(card_data['rating'])
            self.set_folder(card_data['extra']['recent-folder'])
 
        # if it is a direct card to a media
        if self.card_image.get_media_path():
            
            # TODO
            #qp.setBrush( QColor(COLOR_CARD_BORDER_MEDIA))
            
            # Show the RATING section
            self.card_rating.setHidden(False)

        # if it is a Collector
        else:
            
            # TODO
            #qp.setBrush( QColor(COLOR_CARD_BORDER_CONTAINER ))
            
            # Hide the RATING section
            self.card_rating.setHidden(True)
            
            self.add_info_line_stretch()

 
      
    def get_card_holder( self ):
        return self.card_holder

    def set_image_path( self, image_path ):
        self.card_image.set_image_path( image_path )
        
    def set_media_path( self, media_path ):
        self.card_image.set_media_path( media_path )

    def get_media_path( self ):
        return self.card_image.get_media_path( )

    def set_card_path( self, path ):
        self.card_path = path
        
    def get_card_path( self ):
        return self.card_path
    
    def set_sub_cards( self, sub_cards ):
        self.card_image.set_sub_cards( sub_cards )
        
    def get_sub_cards( self ):
        return self.card_image.get_sub_cards()
        
    def set_title(self, title):
        self.card_information.set_title(title)

    def get_title(self):
        return self.card_information.get_title()
        
    def add_separator(self):
        self.card_information.add_separator()
        
    def add_info_line( self, label, value ):
        self.card_information.add_info_line( label, value )
        
    def add_info_line_stretch(self):
        self.card_information.add_stretch()
        
    def add_element_to_collector_line( self, label, value ):
        self.card_information.add_element_to_collector_line( label, value )
        
    def set_rating( self, rating ):
        self.card_rating.set_rating(rating)
        
    def set_folder( self, folder ):
        self.card_rating.set_folder(folder)
        








   
        
        
        



        
        
        
        
# ---------------------------------------------------
#
# CardInfoTitle
#
# ---------------------------------------------------
class CardInfoTitle(QLabel):
    def __init__(self):
        super().__init__()

        self.setWordWrap(True)
        
        # border
        self.setFrameShape(QFrame.Panel)
        self.setFrameShadow(QFrame.Sunken)
        self.setLineWidth(0)

        # font, colors
        self.setFont(QFont( FONT_TYPE, INFO_TITLE_FONT_SIZE, weight=QFont.Bold))
        self.text = None
                
        #self.setAttribute(QtCore.Qt.WA_StyledBackground, True)
        #self.setStyleSheet('background-color: yellow')
    
    def set_title(self, title):
        self.setText(title)
        self.text = title
        
    def get_title(self):
        return self.text
        
        


class CardInfoLineLabel(QLabel):
    def __init__(self, label):
        super().__init__()
 
        #self.setAlignment(Qt.AlignTop);
        self.setMinimumWidth( INFO_LABEL_WIDTH )
        
        # font, colors
        self.setFont(QFont( FONT_TYPE, INFO_FONT_SIZE, weight=QFont.Normal))

        self.setText(label)

class CardInfoLineValue(QLabel):
    def __init__(self, value):
        super().__init__()
        
        #self.setWordWrap(True)
        self.setWordWrap(False)
        #self.setMaximumHeight( ONE_LINE_HEIGHT )
        
        #line_layout = QHBoxLayout(self)        
        #line_layout.setContentsMargins(15, 15, 15, 15)
        #self.setLayout( line_layout )
        
        
        # font, colors
        self.setFont(QFont( FONT_TYPE, INFO_FONT_SIZE, weight=QFont.Normal))
    
        self.setText(value)

# ---------------------------------------------------
#
# CardInfoLine 
#
# It contains a label and value horizontaly ordered
#
# ---------------------------------------------------
class CardInfoLine(QWidget):
    def __init__(self, label, value):
        super().__init__()
    
        line_layout = QHBoxLayout(self)
        line_layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout( line_layout )
        line_layout.setSpacing(0)

        line_layout.addWidget(CardInfoLineLabel(label + ":"),0)
        line_layout.addWidget(CardInfoLineValue(value), 1) 
        

#
# CardInfoLinesHolder
#
# It contains CardInfoLine objects vertically ordered
#
class CardInfoLinesHolder(QWidget):
    def __init__(self):
        super().__init__()
    
        self.line_layout = QHBoxLayout(self)
        self.line_layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout( self.line_layout )
        self.line_layout.setSpacing(1)
        
    def add_element(self, label, value ):
        self.line_layout.addWidget( CardInfoLine( label, value) )

#
# CardCollectorLine
#
class CardCollectorLine(QWidget):
    def __init__(self):
        super().__init__()
    
        self.line_layout = QHBoxLayout(self)
        self.line_layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout( self.line_layout )
        self.line_layout.setSpacing(1)
        
    def add_element(self, label, value ):

        label_l = QLabel(label + ": ")
        label_l.setFont(QFont( FONT_TYPE, INFO_FONT_SIZE, weight=QFont.Normal))
        self.line_layout.addWidget(label_l)
        
        value_l = QLabel(value)
        value_l.setFont(QFont( FONT_TYPE, INFO_FONT_SIZE, weight=QFont.Bold))
        self.line_layout.addWidget(value_l)

        self.line_layout.addStretch(1)

        

# ---------------------------------------------------
#
# CardInformation
#
# It contains two other containers vertically ordered:
# -CardInfoTitle
# -CardInfoLinesHolder
#
# ---------------------------------------------------
class CardInformation(QWidget):
    def __init__(self):
        super().__init__()
        
        self.info_layout = QVBoxLayout(self)
        self.setLayout(self.info_layout)
        self.info_layout.setContentsMargins(0,0,0,0)
        self.info_layout.setSpacing(0)
        
        self.card_info_title = CardInfoTitle()
        self.info_layout.addWidget( self.card_info_title )
        #self.card_info_lines_holder = CardInfoLinesHolder()
        self.card_info_lines_holder = CardCollectorLine()
        self.info_layout.addWidget( self.card_info_lines_holder )

    def set_title(self, title ):
        self.card_info_title.set_title(title)
        
    def get_title(self):
        return self.card_info_title.get_title()
        
    def add_separator(self):
        self.info_layout.addWidget( QHLine() )
        
    def add_info_line( self, label, value ):
        self.info_layout.addWidget( CardInfoLine( label, value ) )
        
    def add_stretch( self ):
        self.info_layout.addStretch(1)

    def add_element_to_collector_line( self, label, value ):
        self.card_info_lines_holder.add_element( label, value )


# ================
#
# Card Image
#
# ================
class CardImage(QWidget):
    def __init__(self, panel ):
        super().__init__()
        
        self.panel = panel
        
        self_layout = QHBoxLayout(self)
        self_layout.setContentsMargins(2,2,2,2)
        self.setLayout( self_layout )
        #p = self.palette()
        #p.setColor(self.backgroundRole(), Qt.red)
        #self.setPalette(p)
        self.setStyleSheet('background: black')

        self.image_panel = QLabel(self)
        self.image_panel.setAlignment(Qt.AlignCenter)
        panel_layout = QHBoxLayout(self.image_panel)
        panel_layout.setContentsMargins(0,0,0,0)
        self_layout.addWidget(self.image_panel)

        self.media_path = None
        self.sub_cards = [] #json.loads('[]')
        
        self.setMinimumWidth(PICTURE_WIDTH)
        self.setMaximumWidth(PICTURE_WIDTH)
        self.setMinimumHeight(PICTURE_HEIGHT)
        self.setMaximumHeight(PICTURE_HEIGHT)
        
        self.mouse_pressed_for_click = False

    # Mouse Hover in
    def enterEvent(self, event):
        self.update()
        QApplication.setOverrideCursor(Qt.PointingHandCursor)
        event.ignore()

    # Mouse Hover out
    def leaveEvent(self, event):
        self.update()
        QApplication.restoreOverrideCursor()
        event.ignore()
    
    def paintEvent(self, event):
        if self.underMouse():                       
            self.setStyleSheet('background: gray')
        else:
            self.setStyleSheet('background: black')
        
        super().paintEvent(event)
        
    def set_image_path( self, image_path ):
        pixmap = QPixmap( image_path )
        if pixmap.width() >= pixmap.height():
            smaller_pixmap = pixmap.scaledToWidth(PICTURE_WIDTH)
        else:
            smaller_pixmap = pixmap.scaledToHeight(PICTURE_WIDTH)
        self.image_panel.setPixmap(smaller_pixmap)        

    def set_media_path( self, media_path ):
        self.media_path = media_path
     
    def get_media_path( self ):
        return self.media_path

    def set_sub_cards( self, sub_cards ):
        self.sub_cards = sub_cards

    def get_sub_cards( self ):        
        return self.sub_cards      
     

    # --------------------
    #
    # Mouse Press on Image
    #
    # --------------------
    def mousePressEvent(self, event):
        if event.buttons() == Qt.LeftButton and self.panel.card.get_status() == Card.STATUS_SELECTED:
            self.mouse_pressed_for_click = True
            event.accept()
        else:
            event.ignore()

    def mouseMoveEvent(self, event):
        if event.buttons() == Qt.LeftButton:
            self.mouse_pressed_for_click = False
            event.accept()
        else:
            event.ignore()
     
    def mouseReleaseEvent(self, event):
        if event.button() != Qt.LeftButton or not self.mouse_pressed_for_click:
            self.mouse_pressed_for_click = False
            event.ignore()
            return
        
        self.mouse_already_pressed = False
        
        media_path = self.get_media_path()
            
        # Play media
        if media_path:

            param_list = []

            # video
            if get_pattern_video().match(self.media_path):
                switch_list = config_ini['media_player_video_param'].split(" ")
                param_list.append(config_ini['media_player_video'])
                param_list += switch_list
                param_list.append(self.media_path)

            # audio
            elif get_pattern_audio().match(self.media_path):
                switch_list = config_ini['media_player_audio_param'].split(" ")
                param_list.append(config_ini['media_player_audio'])
                #param_list += switch_list
                param_list.append(self.media_path)

            thread = Thread(target = call, args = (param_list, ))
            thread.start()
            
        else:
            # go deeper
            self.panel.card_holder.parent.go_down_in_hierarchy(self.get_sub_cards(), self.panel.get_title() )
            
        event.accept()

# ===================================================
#
# CardRating
#
# It contains three elments in vertically ordered:
# -Favorite
# -Best
# -New
#
# ===================================================
class CardRating(QLabel):
    def __init__(self):
        super().__init__()
        
        self.rating_layout = QVBoxLayout(self)
        self.setLayout(self.rating_layout)
        self.rating_layout.setContentsMargins(1,1,1,1)
        self.rating_layout.setSpacing(0)
        #self.setStyleSheet('background: green')
        self.setMinimumWidth(RATE_WIDTH)


        # FAVORITE button
        self.rating_favorite_button = QPushButton()
        self.rating_favorite_button.setFocusPolicy(Qt.NoFocus)
        self.rating_favorite_button.setCheckable(True)        
        rating_favorite_icon = QIcon()
        rating_favorite_icon.addPixmap(QPixmap( resource_filename(__name__,os.path.join("img", IMG_FAVORITE_ON)) ), QIcon.Normal, QIcon.On)
        rating_favorite_icon.addPixmap(QPixmap( resource_filename(__name__,os.path.join("img", IMG_FAVORITE_OFF)) ), QIcon.Normal, QIcon.Off)
        self.rating_favorite_button.clicked.connect(self.rating_favorite_button_on_click)        
        self.rating_favorite_button.setIcon( rating_favorite_icon )
        self.rating_favorite_button.setIconSize(QSize(25,25))
        self.rating_favorite_button.setCursor(QCursor(Qt.PointingHandCursor))
        self.rating_favorite_button.setStyleSheet("background:transparent; border:none") 
        self.rating_layout.addWidget( self.rating_favorite_button )

        # BEST button
        self.rating_best_button = QPushButton()
        self.rating_best_button.setFocusPolicy(Qt.NoFocus)
        self.rating_best_button.setCheckable(True)        
        rating_best_icon = QIcon()
        rating_best_icon.addPixmap(QPixmap( resource_filename(__name__,os.path.join("img", IMG_BEST_ON)) ), QIcon.Normal, QIcon.On)
        rating_best_icon.addPixmap(QPixmap( resource_filename(__name__,os.path.join("img", IMG_BEST_OFF)) ), QIcon.Normal, QIcon.Off)
        self.rating_best_button.clicked.connect(self.rating_best_button_on_click)        
        self.rating_best_button.setIcon( rating_best_icon )
        self.rating_best_button.setIconSize(QSize(25,25))
        self.rating_best_button.setCursor(QCursor(Qt.PointingHandCursor))
        self.rating_best_button.setStyleSheet("background:transparent; border:none") 
        self.rating_layout.addWidget( self.rating_best_button )

        # NEW button
        self.rating_new_button = QPushButton()
        self.rating_new_button.setFocusPolicy(Qt.NoFocus)
        self.rating_new_button.setCheckable(True)        
        rating_new_icon = QIcon()
        rating_new_icon.addPixmap(QPixmap( resource_filename(__name__,os.path.join("img", IMG_NEW_ON)) ), QIcon.Normal, QIcon.On)
        rating_new_icon.addPixmap(QPixmap( resource_filename(__name__,os.path.join("img", IMG_BEST_OFF)) ), QIcon.Normal, QIcon.Off)
        self.rating_new_button.clicked.connect(self.rating_new_button_on_click)        
        self.rating_new_button.setIcon( rating_new_icon )
        self.rating_new_button.setIconSize(QSize(25,25))
        self.rating_new_button.setCursor(QCursor(Qt.PointingHandCursor))
        self.rating_new_button.setStyleSheet("background:transparent; border:none") 
        self.rating_layout.addWidget( self.rating_new_button )
        
        #self.rating_layout.addStretch(1)

    def set_folder(self, folder):
        self.folder = folder

    def set_rating(self, rating):
        self.rating = rating
        self.rating_favorite_button.setChecked(rating[ RATING_KEY_FAVORITE ] == 'y')
        self.rating_best_button.setChecked(rating[ RATING_KEY_BEST ] == 'y')
        self.rating_new_button.setChecked(rating[ RATING_KEY_NEW ] == 'y')

    # -----------------------
    #
    # FAVORITE icon clicked
    #
    # -----------------------
    def rating_favorite_button_on_click(self):
        self.rating[ RATING_KEY_FAVORITE ] = 'y' if self.rating_favorite_button.isChecked() else 'n'
        card_ini = Property( os.path.join(self.folder, FILE_CARD_INI), True )
        card_ini.update(SECTION_RATING, RATING_KEY_FAVORITE, self.rating[RATING_KEY_FAVORITE])

    # -----------------------
    #
    # BEST icon clicked
    #
    # -----------------------       
    def rating_best_button_on_click(self):
        self.rating[RATING_KEY_BEST] = 'y' if self.rating_best_button.isChecked() else 'n'
        card_ini = Property( os.path.join(self.folder, FILE_CARD_INI), True )
        card_ini.update(SECTION_RATING, RATING_KEY_BEST, self.rating[RATING_KEY_BEST])

    # -----------------------
    #
    # NEW icon clicked
    #
    # -----------------------            
    def rating_new_button_on_click(self):
        self.rating[RATING_KEY_NEW] = 'y' if self.rating_new_button.isChecked() else 'n'
        card_ini = Property( os.path.join(self.folder, FILE_CARD_INI), True )
        card_ini.update(SECTION_RATING, RATING_KEY_NEW, self.rating[RATING_KEY_NEW])




from PyQt5.QtWidgets import QWidget
from PyQt5.QtWidgets import QVBoxLayout
from PyQt5.QtWidgets import QHBoxLayout
from PyQt5.QtWidgets import QScrollArea
from PyQt5.QtWidgets import QDesktopWidget
from PyQt5.QtWidgets import QPushButton
from PyQt5.QtWidgets import QLabel
from PyQt5.QtWidgets import QFrame
from PyQt5.QtWidgets import QSpacerItem
from PyQt5.QtWidgets import QSizePolicy
from PyQt5.QtWidgets import QApplication
from PyQt5.QtWidgets import QComboBox
from PyQt5.QtWidgets import QCheckBox
from PyQt5.QtWidgets import QStyleFactory
from PyQt5.QtWidgets import QFileDialog
from PyQt5.QtWidgets import QInputDialog
from PyQt5.QtWidgets import QDialog
from PyQt5.QtWidgets import QDialogButtonBox
from PyQt5.QtWidgets import QLineEdit


from PyQt5.QtGui import QFont
from PyQt5.QtGui import QColor
from PyQt5.QtGui import QPalette
from PyQt5.QtGui import QPixmap
from PyQt5.QtGui import QPainter
from PyQt5.QtGui import QIcon
from PyQt5.QtGui import QCursor
from PyQt5.QtGui import QDrag
from PyQt5.QtGui import QPen
from PyQt5.QtGui import QMouseEvent

from PyQt5 import QtCore

from PyQt5.QtCore import Qt
from PyQt5.QtCore import QSize
from PyQt5.QtCore import QAbstractTableModel
from PyQt5.QtCore import QModelIndex
from PyQt5.QtCore import QVariant
from PyQt5.QtCore import QThread
from PyQt5.QtCore import pyqtSignal
from PyQt5.QtCore import QMimeData
from PyQt5.QtCore import QPoint
from PyQt5.QtCore import QEvent
from PyQt5.QtCore import QObject

from akoteka.handle_property import ConfigIni
from akoteka.handle_property import Dict

class QHLine(QFrame):
    def __init__(self):
        super(QHLine, self).__init__()
        self.setFrameShape(QFrame.HLine)
        self.setFrameShadow(QFrame.Sunken)

class QVLine(QFrame):
    def __init__(self):
        super(QVLine, self).__init__()
        self.setFrameShape(QFrame.VLine)
        self.setFrameShadow(QFrame.Sunken)



from setuptools import setup, find_packages
from akoteka.setup.setup import getSetupIni

sp=getSetupIni()

setup(
      name=sp['name'],
      version=sp['version'],
      description='Videoteka',
      long_description="Records your media contents in searchable form",	#=open('README.md', encoding="utf-8").read(),
      url='http://github.com/dallaszkorben/akoteka',
      author='dallaszkorben',
      author_email='dallaszkorben@gmail.com',
      license='MIT',
      classifiers =[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
      ],
      packages = find_packages(),
      setup_requires=[ "pyqt5", "pyqt5-sip", "numpy", "pyttsx3", 'configparser'],
      install_requires=["pyqt5", 'pyqt5-sip', 'numpy','pyttsx3', 'configparser' ],
      entry_points = {
        'console_scripts':
                ['akoteka=akoteka.gui.main_window:main']
      },
      package_data={
        'cardholder': ['img/*.gif'],
        'akoteka': ['gui/img/*.png'],
        'akoteka': ['setup/setup.ini'],
        'akoteka': ['dict/*.properties']
      },
      include_package_data = True,
      zip_safe=False)